@@ -1,7 +1,7 @@
 /* column.c
  * Routines for handling column preferences
  *
- * $Id: column.c 22038 2007-06-04 20:15:10Z sfisher $
+ * $Id: column.c 24440 2008-02-22 23:59:32Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -48,63 +48,67 @@
 const gchar *
 col_format_to_string(gint fmt) {
   const gchar *slist[] = {
-	"%m", 
-	"%t",
-	"%Rt",
-	"%At",
-	"%Yt",
-	"%Tt",
-	"%Gt",
-	"%s",
-	"%rs",
-	"%us",
-	"%hs",
-	"%rhs",
-	"%uhs",
-	"%ns",
-	"%rns",
-	"%uns",
-	"%d",
-	"%rd",
-	"%ud",
-	"%hd",
-	"%rhd",
+    "%m",
+    "%t",
+    "%Rt",
+    "%At",
+    "%Yt",
+    "%Tt",
+    "%Gt",
+    "%rct",
+    "%dct",
+    "%s",
+    "%rs",
+    "%us",
+    "%hs",
+    "%rhs",
+    "%uhs",
+    "%ns",
+    "%rns",
+    "%uns",
+    "%d",
+    "%rd",
+    "%ud",
+    "%hd",
+    "%rhd",
     "%uhd",
-    "%nd", 
+    "%nd",
     "%rnd",
-	"%und", 
-    "%S", 
-    "%rS", 
-    "%uS", 
-    "%D", 
-    "%rD", 
-    "%uD", 
+    "%und",
+    "%S",
+    "%rS",
+    "%uS",
+    "%D",
+    "%rD",
+    "%uD",
     "%p",
-	"%i", 
-    "%L", 
-    "%B", 
-    "%XO", 
-    "%XR", 
-    "%I", 
-    "%c", 
-    "%Xs", 
-	"%Xd", 
-    "%V", 
-    "%x", 
-    "%e", 
-    "%H", 
-    "%P", 
-    "%y", 
-	"%v",
-	"%q",
-	"%f",
-    "%U",  
+    "%i",
+    "%L",
+    "%B",
+    "%XO",
+    "%XR",
+    "%I",
+    "%c",
+    "%Xs",
+    "%Xd",
+    "%V",
+    "%x",
+    "%e",
+    "%H",
+    "%P",
+    "%y",
+    "%z",
+    "%q",
+    "%f",
+    "%U",
     "%E",
-	"%C",
-	"%l",
-	"%z"
-};
-                     
+    "%C",
+    "%l",
+    "%a",
+    "%F",
+    "%Cus"
+  };
+
   if (fmt < 0 || fmt >= NUM_COL_FMTS)
     return NULL;
 
@@ -121,6 +125,8 @@ static const gchar *dlist[NUM_COL_FMTS] = {
 	"Absolute date and time",                   /* COL_ABS_DATE_TIME */
 	"Delta time",                               /* COL_DELTA_TIME */
 	"Delta time displayed",                     /* COL_DELTA_TIME_DIS */
+	"Relative time (conversation)",             /* COL_REL_CONV_TIME */
+	"Delta time (conversation)",                /* COL_DELTA_CONV_TIME */
 	"Source address",                           /* COL_DEF_SRC */
 	"Src addr (resolved)",                      /* COL_RES_SRC */
 	"Src addr (unresolved)",                    /* COL_UNRES_SRC */
@@ -163,11 +169,14 @@ static const gchar *dlist[NUM_COL_FMTS] = {
 	"DCE/RPC call (cn_call_id / dg_seqnum)",    /* COL_DCE_CALL */
 	"DCE/RPC context ID (cn_ctx_id)",           /* COL_DCE_CTX */
 	"802.1Q VLAN id",                           /* COL_8021Q_VLAN_ID */
-	"IP DSCP Value",		     	    /* COL_DSCP_VALUE */
-	"L2 COS Value (802.1p)",		    /* COL_COS_VALUE */
+	"IP DSCP Value",                            /* COL_DSCP_VALUE */
+	"L2 COS Value (802.1p)",                    /* COL_COS_VALUE */
 	"TEI",                                      /* XXX - why is it missing in column_utils.c and elsewhere? */
-	"Frame Relay DLCI",							/* COL_FR_DLCI */
-	"GPRS BSSGP TLLI",							/* COL_BSSGP_TLLI */
+	"Frame Relay DLCI",                         /* COL_FR_DLCI */
+	"GPRS BSSGP TLLI",                          /* COL_BSSGP_TLLI */
+	"Expert Info Severity",                     /* COL_EXPERT */
+	"Frequency/Channel",                        /* COL_FREQ_CHAN */
+	"Custom"                                    /* COL_CUSTOM */
 };
 
 const gchar *
@@ -286,12 +295,21 @@ get_column_format_matches(gboolean *fmt_list, gint format) {
     case COL_BSSGP_TLLI:
       fmt_list[COL_BSSGP_TLLI] = TRUE;
       break;
+    case COL_EXPERT:
+      fmt_list[COL_EXPERT] = TRUE;
+      break;
+    case COL_FREQ_CHAN:
+      fmt_list[COL_FREQ_CHAN] = TRUE;
+      break;
+    case COL_CUSTOM:
+      fmt_list[COL_CUSTOM] = TRUE;
+      break;
     default:
       break;
   }
 }
 
-/* Returns a string representing the longest possible value for 
+/* Returns a string representing the longest possible value for
    a timestamp column type. */
 static const char *
 get_timestamp_column_longest_string(gint type, gint precision)
@@ -432,7 +450,7 @@ get_timestamp_column_longest_string(gint type, gint precision)
 	return "";
 }
 
-/* Returns the longer string of the column title or the hard-coded width of 
+/* Returns the longer string of the column title or the hard-coded width of
  * its contents for building the packet list layout. */
 const gchar *
 get_column_width_string(gint format, gint col)
@@ -481,6 +499,10 @@ get_column_longest_string(gint format)
     case COL_DELTA_TIME_DIS:
       return get_timestamp_column_longest_string(TS_DELTA_DIS, timestamp_get_precision());
       break;
+    case COL_REL_CONV_TIME:	/* 'abuse' TS_RELATIVE to set the time format */
+    case COL_DELTA_CONV_TIME:	/* for the conversation related time columns */
+      return get_timestamp_column_longest_string(TS_RELATIVE, timestamp_get_precision());
+      break;
     case COL_DEF_SRC:
     case COL_RES_SRC:
     case COL_UNRES_SRC:
@@ -571,6 +593,15 @@ get_column_longest_string(gint format)
     case COL_BSSGP_TLLI:
       return "0xffffffff";
       break;
+    case COL_EXPERT:
+      return "ERROR";
+      break;
+    case COL_FREQ_CHAN:
+      return "9999 MHz [A 999]";
+      break;
+    case COL_CUSTOM:
+      return "0000000000";	/* not the longest, but the longest is too long */
+      break;
     default: /* COL_INFO */
       return "Source port: kerberos-master  Destination port: kerberos-master";
       break;
@@ -585,21 +616,6 @@ get_column_char_width(gint format)
   return strlen(get_column_longest_string(format));
 }
 
-#define TIME_DEF      0
-#define TIME_REL      1
-#define TIME_ABS      2
-#define DATE_TIME_ABS 3
-#define TIME_DEL      4
-#define TIME_DEL_DIS  5
-
-#define RES_DEF  0
-#define RES_DO   1
-#define RES_DONT 2
-
-#define ADDR_DEF 0
-#define ADDR_DL  3
-#define ADDR_NET 6
-
 gint
 get_column_format(gint col) {
   GList    *clp = g_list_nth(prefs.col_list, col);
@@ -612,136 +628,11 @@ get_column_format(gint col) {
 
 gint
 get_column_format_from_str(gchar *str) {
-  gchar *cptr = str;
-  gint      res_off = RES_DEF, addr_off = ADDR_DEF, time_off = TIME_DEF;
-  gint      prev_code = -1;
+  gint i;
 
-  /* To do: Make this parse %-formatted strings "for real" */
-  while (*cptr != '\0') {
-    switch (*cptr) {
-      case 't':  /* To do: fix for absolute and delta */
-        return COL_CLS_TIME + time_off;
-        break;
-      case 'm':
-        return COL_NUMBER;
-        break;
-      case 's':
-        if (prev_code == COL_OXID) {
-          return COL_SRCIDX;
-        }
-        else {
-          return COL_DEF_SRC + res_off + addr_off;
-        }
-        break;
-      case 'd':
-        if (prev_code == COL_OXID) {
-          return COL_DSTIDX;
-        }
-        else {
-          return COL_DEF_DST + res_off + addr_off;
-        }
-        break;
-      case 'S':
-        return COL_DEF_SRC_PORT + res_off;
-        break;
-      case 'D':
-        return COL_DEF_DST_PORT + res_off;
-        break;
-      case 'p':
-        return COL_PROTOCOL;
-        break;
-      case 'i':
-        return COL_INFO;
-        break;
-      case 'r':
-        res_off = RES_DO;
-        break;
-      case 'u':
-        res_off = RES_DONT;
-        break;
-      case 'h':
-        addr_off = ADDR_DL;
-        break;
-      case 'n':
-        addr_off = ADDR_NET;
-        break;
-      case 'R':
-        if (prev_code == COL_OXID) {
-            return COL_RXID;
-        }
-        else {
-            time_off = TIME_REL;
-        }
-        break;
-      case 'A':
-        time_off = TIME_ABS;
-        break;
-      case 'Y':
-        time_off = DATE_TIME_ABS;
-        break;
-      case 'T':
-        time_off = TIME_DEL;
-        break;
-      case 'G': /* Todo: use a better letter for time since last displayed packet */
-        time_off = TIME_DEL_DIS;
-        break;
-      case 'L':
-        return COL_PACKET_LENGTH;
-        break;
-      case 'B':
-        return COL_CUMULATIVE_BYTES;
-        break;
-      case 'X':
-        prev_code = COL_OXID;
-        break;
-      case 'O':
-        return COL_OXID;
-        break;
-      case 'I':
-        return COL_IF_DIR;
-        break;
-      case 'c':
-        return COL_CIRCUIT_ID;
-        break;
-      case 'V':
-        return COL_VSAN;
-        break;
-      case 'x':
-        return COL_TX_RATE;
-        break;
-      case 'e':
-        return COL_RSSI;
-        break;
-      case 'H':
-	return COL_HPUX_SUBSYS;
-	break;
-      case 'P':
-	return COL_HPUX_DEVID;
-	break;
-      case 'y':
-	return COL_DCE_CALL;
-	break;
-      case 'z':
-	return COL_DCE_CTX;
-	break;
-      case 'q':
-	return COL_8021Q_VLAN_ID;
-	break;
-      case 'f':
-        return COL_DSCP_VALUE;
-        break;
-      case 'U':
-        return COL_COS_VALUE;
-        break;
-      case 'E':
-	return COL_TEI;
-      case 'C':
-	return COL_FR_DLCI;
-      case 'l':
-	return COL_BSSGP_TLLI;
-	break;
-    }
-    cptr++;
+  for (i = 0; i < NUM_COL_FMTS; i++) {
+    if (strcmp(str, col_format_to_string(i)) == 0)
+      return i;
   }
   return -1;	/* illegal */
 }
@@ -785,12 +676,12 @@ build_column_format_array(capture_file *cfile, gboolean reset_fences)
     cfile->cinfo.col_expr_val[i] = (gchar *) g_malloc(sizeof(gchar) *
 						     COL_MAX_LEN);
   }
-    
+
   for (i = 0; i < cfile->cinfo.num_cols; i++) {
     for (j = 0; j < NUM_COL_FMTS; j++) {
       if (!cfile->cinfo.fmt_matx[i][j])
 	      continue;
-		    
+
       if (cfile->cinfo.col_first[j] == -1)
         cfile->cinfo.col_first[j] = i;
 
