@@ -1,7 +1,7 @@
 /* conversation.c
  * Routines for building lists of packets that are part of a "conversation"
  *
- * $Id: conversation.c 18825 2006-08-02 19:50:04Z etxrab $
+ * $Id: conversation.c 23303 2007-10-29 06:54:25Z sake $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -187,17 +187,11 @@ conversation_hash_exact(gconstpointer v)
 {
 	const conversation_key *key = (const conversation_key *)v;
 	guint hash_val;
-	int i;
 
 	hash_val = 0;
-	for (i = 0; i < key->addr1.len; i++)
-		hash_val += key->addr1.data[i];
-
+	ADD_ADDRESS_TO_HASH(hash_val, &key->addr1);
 	hash_val += key->port1;
-
-	for (i = 0; i < key->addr2.len; i++)
-		hash_val += key->addr2.data[i];
-
+	ADD_ADDRESS_TO_HASH(hash_val, &key->addr2);
 	hash_val += key->port2;
 
 	return hash_val;
@@ -264,14 +258,10 @@ conversation_hash_no_addr2(gconstpointer v)
 {
 	const conversation_key *key = (const conversation_key *)v;
 	guint hash_val;
-	int i;
 
 	hash_val = 0;
-	for (i = 0; i < key->addr1.len; i++)
-		hash_val += key->addr1.data[i];
-
+	ADD_ADDRESS_TO_HASH(hash_val, &key->addr1);
 	hash_val += key->port1;
-
 	hash_val += key->port2;
 
 	return hash_val;
@@ -322,16 +312,11 @@ conversation_hash_no_port2(gconstpointer v)
 {
 	const conversation_key *key = (const conversation_key *)v;
 	guint hash_val;
-	int i;
 
 	hash_val = 0;
-	for (i = 0; i < key->addr1.len; i++)
-		hash_val += key->addr1.data[i];
-
+	ADD_ADDRESS_TO_HASH(hash_val, &key->addr1);
 	hash_val += key->port1;
-
-	for (i = 0; i < key->addr2.len; i++)
-		hash_val += key->addr2.data[i];
+	ADD_ADDRESS_TO_HASH(hash_val, &key->addr2);
 
 	return hash_val;
 }
@@ -381,12 +366,9 @@ conversation_hash_no_addr2_or_port2(gconstpointer v)
 {
 	const conversation_key *key = (const conversation_key *)v;
 	guint hash_val;
-	int i;
 
 	hash_val = 0;
-	for (i = 0; i < key->addr1.len; i++)
-		hash_val += key->addr1.data[i];
-
+	ADD_ADDRESS_TO_HASH(hash_val, &key->addr1);
 	hash_val += key->port1;
 
 	return hash_val;
@@ -674,7 +656,7 @@ conversation_lookup_hashtable(GHashTable *hashtable, guint32 frame_num, address
 
 	if (match) {
 		for (conversation = match->next; conversation; conversation = conversation->next) {
-			if ((conversation->setup_frame < frame_num)
+			if ((conversation->setup_frame <= frame_num)
 				&& (conversation->setup_frame > match->setup_frame))
 				match = conversation;
 		}
