@@ -2,7 +2,7 @@
  * Routines for BACnet (APDU) dissection
  * Copyright 2004, Herbert Lischka <lischka@kieback-peter.de>, Berlin
  *
- * $Id: packet-bacapp.h 19410 2006-10-03 18:11:55Z etxrab $
+ * $Id: packet-bacapp.h 24169 2008-01-23 19:47:40Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -1802,10 +1802,11 @@ fSpecialEvent (tvbuff_t *tvb, proto_tree *tree, guint offset);
  * @param tvb
  * @param tree
  * @param offset
+ * @param label
  * @return modified offset
  */
 static guint
-fTimeStamp (tvbuff_t *tvb, proto_tree *tree, guint offset);
+fTimeStamp (tvbuff_t *tvb, proto_tree *tree, guint offset, const gchar *label);
 
 /**
  * BACnetTimeValue ::= SEQUENCE {
@@ -1845,7 +1846,7 @@ fVTSession (tvbuff_t *tvb, proto_tree *tree, guint offset);
  * -- 4 = days numbered 22-28
  * -- 5 = days numbered 29-31
  * -- 6 = last 7 days of this month
- * -- XFF = any week of this month
+ * -- X'FF' = any week of this month
  * -- third octet dayOfWeek (1..7) where 1 = Monday
  * -- 7 = Sunday
  * -- X'FF' = any day of week
