@@ -5,7 +5,7 @@
  * Copyright : 2005 Viorel Suman <vsuman[AT]avmob.ro>, Lucian Piros <lpiros[AT]avmob.ro>
  *             In association with Avalanche Mobile BV, http://www.avmob.com
  *
- * $Id: packet-cimd.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-cimd.c 23252 2007-10-23 05:50:00Z etxrab $
  *
  * Refer to the AUTHORS file or the AUTHORS section in the man page
  * for contacting the author(s) of this file.
@@ -285,7 +285,7 @@ static void dissect_cimd_ud(tvbuff_t *tvb, proto_tree *tree, gint pindex, gint s
   gint g_offset, g_size;
   gchar token[4];  
   gchar ch;
-  char* mapping[128] = {
+  const char* mapping[128] = {
     "_Oa","_L-", "", "_Y-", "_e`", "_e'", "_u`", "_i`", "_o`","_C,", /*10*/
     "", "_O/", "_o/", "", "_A*", "_a*","_gd", "_--", "_gf", "_gg", "_gl", /*21*/
     "_go", "_gp","_gi", "_gs", "_gt", "_gx", "_XX","_AE","_ae", "_ss","_E'", /*32*/
@@ -688,7 +688,7 @@ dissect_cimd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   {
     col_set_str(pinfo->cinfo, COL_INFO, "");
     if (checksumIsValid)
-      col_append_fstr(pinfo->cinfo, COL_INFO, "%s", match_strval(OC, vals_hdr_OC));
+      col_append_str(pinfo->cinfo, COL_INFO, match_strval(OC, vals_hdr_OC));
     else
       col_append_fstr(pinfo->cinfo, COL_INFO, "%s - %s", match_strval(OC, vals_hdr_OC), "invalid checksum");
   }
