@@ -1,7 +1,7 @@
 /* packet-clnp.c
  * Routines for ISO/OSI network and transport protocol packet disassembly
  *
- * $Id: packet-clnp.c 23405 2007-11-08 22:40:20Z etxrab $
+ * $Id: packet-clnp.c 24286 2008-02-07 15:44:45Z stig $
  * Laurent Deniel <laurent.deniel@free.fr>
  * Ralf Schneider <Ralf.Schneider@t-online.de>
  *
@@ -845,8 +845,6 @@ static int ositp_decode_DR(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
     case (2):     str = "Session entity not attached to TSAP"; break;
     case (3):     str = "Address unknown"; break;
     default:      return -1;
-      /*NOTREACHED*/
-      break;
   }
 
   if (check_col(pinfo->cinfo, COL_INFO))
@@ -961,8 +959,6 @@ static int ositp_decode_DT(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 
     default : /* bad TPDU */
       return -1;
-      /*NOTREACHED*/
-      break;
   }
 
   pinfo->clnp_dstref = dst_ref;
@@ -1155,8 +1151,6 @@ static int ositp_decode_ED(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 
     default : /* bad TPDU */
       return -1;
-      /*NOTREACHED*/
-      break;
   } /* li */
 
   dst_ref = tvb_get_ntohs(tvb, offset + P_DST_REF);
@@ -1234,8 +1228,6 @@ static int ositp_decode_RJ(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
       break;
     default :
       return -1;
-      /*NOTREACHED*/
-      break;
   }
 
   dst_ref = tvb_get_ntohs(tvb, offset + P_DST_REF);
@@ -1571,8 +1563,6 @@ static int ositp_decode_EA(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
 
     default : /* bad TPDU */
       return -1;
-      /*NOTREACHED*/
-      break;
   } /* li */
 
   dst_ref = tvb_get_ntohs(tvb, offset + P_DST_REF);
@@ -1650,8 +1640,6 @@ static int ositp_decode_ER(tvbuff_t *tvb, int offset, guint8 li, guint8 tpdu,
       break;
     default:
       return -1;
-      /*NOTREACHED*/
-      break;
   }
 
   dst_ref = tvb_get_ntohs(tvb, offset + P_DST_REF);
@@ -1917,11 +1905,11 @@ static void dissect_clnp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				"Unknown (0x%02x)");
   flag_string[0] = '\0';
   if (cnf_type & CNF_SEG_OK)
-    strcat(flag_string, "S ");
+    strncat(flag_string, "S ", 3);
   if (cnf_type & CNF_MORE_SEGS)
-    strcat(flag_string, "M ");
+    strncat(flag_string, "M ", 3);
   if (cnf_type & CNF_ERR_OK)
-    strcat(flag_string, "E ");
+    strncat(flag_string, "E ", 3);
   if (tree) {
     ti = proto_tree_add_uint_format(clnp_tree, hf_clnp_type, tvb, P_CLNP_TYPE, 1,
 			       cnf_type,
