@@ -4,7 +4,7 @@
  *
  *  (c) 2006, Luis E. Garcia Ontanon <luis.ontanon@gmail.com>
  *
- * $Id: packet-h248_annex_c.c 21773 2007-05-15 05:49:43Z etxrab $
+ * $Id: packet-h248_annex_c.c 24009 2008-01-04 12:53:29Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -742,7 +742,7 @@ static void dissect_h248_annexc_acodec(proto_tree* tree,
 	asn1_ctx_init(&asn1_ctx, ASN1_ENC_BER, TRUE, pinfo);
 	dissect_ber_octet_string(implicit_p ? *((gboolean*)implicit_p) : FALSE, &asn1_ctx, tree, tvb, 0, hfid, &new_tvb);
 	
-	tree = proto_item_add_subtree(get_ber_last_created_item(),ett_codec);
+	tree = proto_item_add_subtree(asn1_ctx.created_item,ett_codec);
 	len = tvb_length(new_tvb);
 	dissect_codec_mode(tree,new_tvb,0,len);
 }
@@ -776,7 +776,7 @@ static void dissect_h248_annexc_NSAP(proto_tree* tree,
 	asn1_ctx_init(&asn1_ctx, ASN1_ENC_BER, TRUE, pinfo);
 	dissect_ber_octet_string(implicit_p ? *((gboolean*)implicit_p) : FALSE, &asn1_ctx, tree, tvb, 0, hfid, &new_tvb);
 	if (new_tvb) {
-		dissect_nsap(new_tvb, 0,tvb_length_remaining(new_tvb, 0), tree);
+		dissect_nsap(new_tvb, 0,tvb_length(new_tvb), tree);
 		if ( h248_info->term && ! h248_info->term->nsap) {
 			h248_info->term->nsap = se_strdup(tvb_bytes_to_str(new_tvb,0,tvb_length(new_tvb)));
 		}
