@@ -1,19 +1,20 @@
 /* packet-jxta.c
+ *
  * Routines for JXTA packet dissection
- * Copyright 2004-06, Mike Duigou <bondolo@jxta.org>
+ * JXTA specification from https://jxta-spec.dev.java.net
+ *
+ * Copyright 2004-08, Mike Duigou <bondolo@dev.java.net>
+ *
  * Heavily based on packet-jabber.c, which in turn is heavily based on
  * on packet-acap.c, which in turn is heavily based on
  * packet-imap.c, Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
+ * Copied from packet-pop.c, packet-jabber.c, packet-udp.c, packet-http.c
  *
- * $Id: packet-jxta.c 23631 2007-11-28 00:08:20Z gerald $
+ * $Id: packet-jxta.c 24224 2008-01-29 23:28:10Z bondolo $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
- * Copyright 1998 Gerald Combs
- *
- * Copied from packet-pop.c, packet-jabber.c, packet-udp.c, packet-http.c
- *
- * JXTA specification from http://spec.jxta.org
+ * Copyright 2000 Gerald Combs
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -670,10 +671,6 @@ static int dissect_jxta_udp(tvbuff_t * tvb, packet_info * pinfo, proto_tree * tr
             break;
         }
 
-        /* Redo header processing, this time populating the tree. */
-        processed = dissect_jxta_message_framing(jxta_message_framing_tvb, pinfo, tree, &content_length, &content_type);
-
-
         offset += processed;
 
         available = tvb_reported_length_remaining(tvb, offset);
@@ -710,20 +707,13 @@ static int dissect_jxta_udp(tvbuff_t * tvb, packet_info * pinfo, proto_tree * tr
         guint64 content_length = -1;
         gchar *content_type = NULL;
         tvbuff_t *jxta_message_tvb;
-        gint processed = 0;
 
         proto_tree_add_item(jxta_udp_tree, hf_jxta_udpsig, tvb, tree_offset, sizeof(JXTA_UDP_SIG), FALSE);
         tree_offset += sizeof(JXTA_UDP_SIG);
 
         jxta_message_framing_tvb = tvb_new_subset(tvb, tree_offset, -1, -1);
-        processed = dissect_jxta_message_framing(jxta_message_framing_tvb, pinfo, NULL, &content_length, &content_type);
 
-        if ((0 == processed) || (NULL == content_type) || (content_length <= 0) || (content_length > UINT_MAX)) {
-            /** Buffer did not begin with valid framing headers */
-            return 0;
-        }
-
-        tree_offset += dissect_jxta_message_framing(jxta_message_framing_tvb, pinfo, tree, &content_length, &content_type);
+        tree_offset += dissect_jxta_message_framing(jxta_message_framing_tvb, pinfo, jxta_tree, &content_length, &content_type);
 
         jxta_message_tvb = tvb_new_subset(tvb, tree_offset, (gint) content_length, (gint) content_length);
 
