@@ -5,7 +5,7 @@
  * Copyright 2005
  * Written by Martin Mathieson
  *
- * $Id: packet-ms-mms.c 18811 2006-08-01 17:20:13Z martinm $
+ * $Id: packet-ms-mms.c 23761 2007-12-05 09:31:05Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -322,7 +322,7 @@ static gint dissect_msmms_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tre
     /* Work out what type of packet this is and dissect it as such */
 
     /* Just don't dissect if can't even read command signature */
-    if (tvb_length_remaining(tvb, 0) < 8)
+    if (tvb_length(tvb) < 8)
     {
         return 0;
     }
@@ -612,7 +612,7 @@ gint dissect_msmms_data_udp_command(tvbuff_t *tvb, packet_info *pinfo, proto_tre
 
     if (check_col(pinfo->cinfo, COL_INFO))
     {
-        col_add_str(pinfo->cinfo, COL_INFO, "Request to resend packet(s):");
+        col_set_str(pinfo->cinfo, COL_INFO, "Request to resend packet(s):");
     }
 
     /* Show list of packets to resend */
@@ -771,7 +771,7 @@ void dissect_client_transport_info(tvbuff_t *tvb, packet_info *pinfo, proto_tree
 {
     char    *transport_info;
     guint   ipaddr[4];
-    char    protocol[3];
+    char    protocol[3+1];
     guint   port;
     int     fields_matched;
 
