@@ -6,7 +6,7 @@
  * Copyright 2000, Philips Electronics N.V.
  * Written by Andreas Sikkema <h323@ramdyne.nl>
  *
- * $Id: packet-rtp.c 23337 2007-11-02 07:43:49Z kukosa $
+ * $Id: packet-rtp.c 24045 2008-01-09 16:21:03Z martinm $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -82,7 +82,7 @@
 /* #define DEBUG_FRAGMENTS   1 */
 
 typedef struct _rfc2198_hdr {
-  guint8 pt;
+  unsigned int pt;
   int offset;
   int len;
   struct _rfc2198_hdr *next;
@@ -146,6 +146,8 @@ static int rtp_tap = -1;
 static dissector_table_t rtp_pt_dissector_table;
 static dissector_table_t rtp_dyn_pt_dissector_table;
 
+static dissector_table_t rtp_hdr_ext_dissector_table;
+
 /* RTP header fields             */
 static int proto_rtp           = -1;
 static int hf_rtp_version      = -1;
@@ -391,6 +393,10 @@ void srtp_add_address(packet_info *pinfo,
 		return;
 	}
 
+#ifdef DEBUG
+	printf("#%u: %srtp_add_address(%s, %u, %u, %s, %u\n", pinfo->fd->num, (srtp_info)?"s":"", address_to_str(addr), port, other_port, setup_method, setup_frame_number);
+#endif
+
 	SET_ADDRESS(&null_addr, AT_NONE, 0, NULL);
 
 	/*
@@ -824,15 +830,21 @@ dissect_rtp_rfc2198(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	proto_tree *rfc2198_hdr_tree = NULL;
 	rfc2198_hdr *hdr_last, *hdr_new;
 	rfc2198_hdr *hdr_chain = NULL;
+	struct _rtp_conversation_info *p_conv_data= NULL;
+	gchar *payload_type_str;
 
-	/* Add try to RFC2198 data */
-	ti = proto_tree_add_text(tree, tvb, offset, -1, "RFC2198: Redundant Audio Data");
+	/* Retrieve RTPs idea of a converation */
+	p_conv_data = p_get_proto_data(pinfo->fd, proto_rtp);
+
+	/* Add try to RFC 2198 data */
+	ti = proto_tree_add_text(tree, tvb, offset, -1, "RFC 2198: Redundant Audio Data");
 	rfc2198_tree = proto_item_add_subtree(ti, ett_rtp_rfc2198);
 
 	hdr_last = NULL;
 	cnt = 0;
 	while (hdr_follow) {
 		cnt++;
+		payload_type_str = NULL;
 
 		/* Allocate and fill in header */
 		hdr_new = ep_alloc(sizeof(rfc2198_hdr));
@@ -841,12 +853,21 @@ dissect_rtp_rfc2198(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		hdr_new->pt = RTP_PAYLOAD_TYPE(octet1);
 		hdr_follow = (octet1 & 0x80);
 
+		/* if it is dynamic payload, let use the conv data to see if it is defined */
+		if ((hdr_new->pt > 95) && (hdr_new->pt < 128)) {
+			if (p_conv_data && p_conv_data->rtp_dyn_payload){
+				payload_type_str = g_hash_table_lookup(p_conv_data->rtp_dyn_payload, &hdr_new->pt);
+			}
+		}
 		/* Add a subtree for this header and add items */
 		ti = proto_tree_add_text(rfc2198_tree, tvb, offset, (hdr_follow)?4:1, "Header %u", cnt);
 		rfc2198_hdr_tree = proto_item_add_subtree(ti, ett_rtp_rfc2198_hdr);
 		proto_tree_add_item(rfc2198_hdr_tree, hf_rtp_rfc2198_follow, tvb, offset, 1, FALSE );
-		proto_tree_add_item(rfc2198_hdr_tree, hf_rtp_payload_type, tvb, offset, 1, FALSE );
-		proto_item_append_text(ti, ": PT=%s", val_to_str(hdr_new->pt, rtp_payload_type_vals, "Unknown (%u)"));
+		proto_tree_add_uint_format(rfc2198_hdr_tree, hf_rtp_payload_type, tvb,
+		    offset, 1, octet1, "Payload type: %s (%u)",
+			payload_type_str ? payload_type_str : val_to_str(hdr_new->pt, rtp_payload_type_vals, "Unknown"),
+			hdr_new->pt);
+		proto_item_append_text(ti, ": PT=%s", payload_type_str ? payload_type_str : val_to_str(hdr_new->pt, rtp_payload_type_vals, "Unknown (%u)"));
 		offset += 1;
 
 		/* Timestamp offset and block length don't apply to last header */
@@ -888,6 +909,7 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	proto_item *ti            = NULL;
 	proto_tree *rtp_tree      = NULL;
 	proto_tree *rtp_csrc_tree = NULL;
+	proto_tree *rtp_hext_tree = NULL;
 	guint8      octet1, octet2;
 	unsigned int version;
 	gboolean    padding_set;
@@ -910,6 +932,7 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	struct _rtp_conversation_info *p_conv_data = NULL;
 	struct srtp_info *srtp_info = NULL;
 	unsigned int srtp_offset;
+	tvbuff_t *newtvb = NULL;
 
 	/* Can tap up to 4 RTP packets within same packet */
 	static struct _rtp_info rtp_info_arr[4];
@@ -989,7 +1012,7 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	rtp_info->info_seq_num = seq_num;
 	rtp_info->info_timestamp = timestamp;
 	rtp_info->info_sync_src = sync_src;
-    rtp_info->info_is_srtp = FALSE;
+	rtp_info->info_is_srtp = FALSE;
 	rtp_info->info_setup_frame_num = 0;
 	rtp_info->info_payload_type_str = NULL;
 
@@ -1031,7 +1054,7 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	p_conv_data = p_get_proto_data(pinfo->fd, proto_rtp);
 
 	if (p_conv_data && p_conv_data->srtp_info) is_srtp = TRUE;
-    rtp_info->info_is_srtp = is_srtp;
+	rtp_info->info_is_srtp = is_srtp;
 
 	if ( check_col( pinfo->cinfo, COL_PROTOCOL ) )   {
 		col_set_str( pinfo->cinfo, COL_PROTOCOL, (is_srtp) ? "SRTP" : "RTP" );
@@ -1141,23 +1164,23 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		offset += 2;
 
 		hdr_extension = tvb_get_ntohs( tvb, offset );
-		if ( tree ) proto_tree_add_uint( rtp_tree, hf_rtp_length, tvb,
-		    offset, 2, hdr_extension);
+		if ( tree ) proto_tree_add_uint( rtp_tree, hf_rtp_length, tvb, offset, 2, hdr_extension);
 		offset += 2;
 		if ( hdr_extension > 0 ) {
 			if ( tree ) {
-				ti = proto_tree_add_item(rtp_tree, hf_rtp_hdr_exts, tvb, offset, hdr_extension * 4,
-				                         FALSE);
-				/* I'm re-using the old tree variable here
-				   from the CSRC list!*/
-				rtp_csrc_tree = proto_item_add_subtree( ti,
-				    ett_hdr_ext );
+				ti = proto_tree_add_item(rtp_tree, hf_rtp_hdr_exts, tvb, offset, hdr_extension * 4, FALSE);
+				rtp_hext_tree = proto_item_add_subtree( ti, ett_hdr_ext );
 			}
-			for (i = 0; i < hdr_extension; i++ ) {
-				if ( tree ) proto_tree_add_uint(rtp_csrc_tree, hf_rtp_hdr_ext, tvb, offset, 4,
-				                                tvb_get_ntohl( tvb, offset ) );
-				offset += 4;
+
+			/* pass interpretation of header extension to a registered subdissector */
+			newtvb = tvb_new_subset(tvb, offset, hdr_extension * 4, hdr_extension * 4);
+			if ( !(rtp_info->info_payload_type_str && dissector_try_string(rtp_hdr_ext_dissector_table,
+				rtp_info->info_payload_type_str, newtvb, pinfo, rtp_hext_tree)) ) {
+				for ( i = 0; i < hdr_extension; i++ ) {
+					if ( tree ) proto_tree_add_uint( rtp_hext_tree, hf_rtp_hdr_ext, tvb, offset, 4, tvb_get_ntohl( tvb, offset ) );
+				}
 			}
+			offset += hdr_extension * 4;
 		}
 	}
 
@@ -1840,6 +1863,9 @@ proto_register_rtp(void)
 												    "Dynamic RTP payload type", FT_STRING, BASE_NONE);
 
 
+	rtp_hdr_ext_dissector_table = register_dissector_table("rtp_hdr_ext", 
+													"RTP header extension", FT_STRING, BASE_NONE);
+
 	rtp_module = prefs_register_protocol(proto_rtp, proto_reg_handoff_rtp);
 
 	prefs_register_bool_preference(rtp_module, "show_setup_info",
@@ -1890,6 +1916,8 @@ proto_reg_handoff_rtp(void)
 
 	dissector_add_handle("udp.port", rtp_handle);
 
+	dissector_add_string("rtp_dyn_payload_type", "red", rtp_rfc2198_handle);
+
   	if (rtp_prefs_initialized) {
 		dissector_delete("rtp.pt", rtp_saved_rfc2198_pt, rtp_rfc2198_handle);
 	} else {
