@@ -3,7 +3,7 @@
  *  
  * (c) 2006, Luis E. Garcia Ontanon <luis.ontanon@gmail.com>
  * 
- * $Id: wslua_util.c 20330 2007-01-05 15:09:33Z lego $
+ * $Id: wslua_util.c 24089 2008-01-14 16:40:23Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -165,7 +165,7 @@ const char* wslua_get_actual_filename(const char* fname) {
 		return fname_clean;
 	}
 	
-	filename = get_persconffile_path(fname_clean,FALSE);
+	filename = get_persconffile_path(fname_clean,FALSE,FALSE);
 	
 	if ( file_exists(filename) ) {
 		return filename;
@@ -179,7 +179,7 @@ const char* wslua_get_actual_filename(const char* fname) {
 WSLUA_FUNCTION wslua_loadfile(lua_State* L) {
 	/* Lua's loadfile() has been modified so that if a file does not exist
 	in the current directory it will look for it in wireshark's user and system directories */
-#define WSLUA_ARG_loadfile_FILENAME 1
+#define WSLUA_ARG_loadfile_FILENAME 1 /* name of the file to be loaded */
 	const char *given_fname = luaL_checkstring(L, WSLUA_ARG_loadfile_FILENAME);
 	const char* filename;
 	
@@ -199,7 +199,7 @@ WSLUA_FUNCTION wslua_loadfile(lua_State* L) {
 WSLUA_FUNCTION wslua_dofile(lua_State* L) {
 	/* Lua's dofile() has been modified so that if a file does not exist
 	in the current directory it will look for it in wireshark's user and system directories */
-#define WSLUA_ARG_dofile_FILENAME 1
+#define WSLUA_ARG_dofile_FILENAME 1 /* name of the file to be run */
 	const char *given_fname = luaL_checkstring(L, WSLUA_ARG_dofile_FILENAME);
 	const char* filename;
 	int n;
@@ -220,7 +220,7 @@ WSLUA_FUNCTION wslua_dofile(lua_State* L) {
 WSLUA_FUNCTION wslua_persconffile_path(lua_State* L) {
 #define WSLUA_OPTARG_persconffile_path_FILENAME 1 /* a filename */
 	const char *fname = luaL_optstring(L, WSLUA_OPTARG_persconffile_path_FILENAME,"");
-	const char* filename = get_persconffile_path(fname,FALSE);
+	const char* filename = get_persconffile_path(fname,FALSE,FALSE);
 	
 	lua_pushstring(L,filename);
 	WSLUA_RETURN(1); /* the full pathname for a file in the personal configuration directory */
