@@ -1,7 +1,7 @@
 /* capture_file_dlg.c
  * Dialog boxes for handling capture files
  *
- * $Id: capture_file_dlg.c 21990 2007-05-29 18:52:42Z guy $
+ * $Id: capture_file_dlg.c 24204 2008-01-26 17:04:49Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -59,6 +59,7 @@
 #include "range_utils.h"
 #include "merge.h"
 #include "util.h"
+#include <wiretap/file_util.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -683,6 +684,13 @@ file_open_ok_cb(GtkWidget *w, gpointer fs) {
     if (rfcode != NULL)
       dfilter_free(rfcode);
     g_free(cf_name);
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    /* XXX - as we cannot start a new event loop (using gtk_dialog_run()),
+     * as this will prevent the user from closing the now existing error
+     * message, simply close the dialog (this is the best we can do here). */	
+    if (file_open_w)
+      window_destroy(file_open_w);
+#endif
     return;
   }
 
@@ -1080,6 +1088,13 @@ file_merge_ok_cb(GtkWidget *w, gpointer fs) {
     if (rfcode != NULL)
       dfilter_free(rfcode);
     g_free(tmpname);
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    /* XXX - as we cannot start a new event loop (using gtk_dialog_run()),
+     * as this will prevent the user from closing the now existing error
+     * message, simply close the dialog (this is the best we can do here). */	
+    if (file_open_w)
+      window_destroy(file_open_w);
+#endif
     return;
   }
   g_free(tmpname);
@@ -1185,11 +1200,11 @@ set_file_type_list(GtkWidget *option_menu)
   GtkWidget *ft_menu, *ft_menu_item;
   int ft;
   guint index;
-  guint item_to_select;
+  gint item_to_select;
 
   /* Default to the first supported file type, if the file's current
      type isn't supported. */
-  item_to_select = 0;
+  item_to_select = -1;
 
   ft_menu = gtk_menu_new();
 
@@ -1212,7 +1227,21 @@ set_file_type_list(GtkWidget *option_menu)
   }
 
   gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), ft_menu);
-  gtk_option_menu_set_history(GTK_OPTION_MENU(option_menu), item_to_select);
+  if (item_to_select >= 0) {
+	  /* Select the current File format in the menu */
+	  gtk_option_menu_set_history(GTK_OPTION_MENU(option_menu), item_to_select);
+	  select_file_type_cb(NULL, GINT_TO_POINTER(filetype));
+  } else {
+	  
+	  /*
+	   * Manually call the signal handler to activate the first menu item
+	   * since gtk_option_menu_set_history() doesn't do it for us. The first two
+	   * entries in the wiretap file types are placeholders so we start at #2, which
+	   * is the normal libpcap format.
+	   */
+	  gtk_option_menu_set_history(GTK_OPTION_MENU(option_menu), 0);
+	  select_file_type_cb(NULL, GINT_TO_POINTER(WTAP_FILE_PCAP));
+  }
 }
 
 static void
@@ -1224,7 +1253,8 @@ select_file_type_cb(GtkWidget *w _U_, gpointer data)
   if (filetype != new_filetype) {
     filetype = new_filetype;
     compressed_cb = OBJECT_GET_DATA(file_save_as_w, "compressed");
-    gtk_widget_set_sensitive(compressed_cb, wtap_dump_can_compress(new_filetype));
+    if(compressed_cb)
+	    gtk_widget_set_sensitive(compressed_cb, wtap_dump_can_compress(new_filetype));
   }
 }
 
@@ -1473,10 +1503,19 @@ static void file_save_as_exists_answered_cb(gpointer dialog _U_, gint btn, gpoin
     switch(btn) {
     case(ESD_BTN_OK):
         /* save file */
-        unlink(cf_name);
+        eth_unlink(cf_name);
         file_save_as_cb(NULL, data);
         break;
     case(ESD_BTN_CANCEL):
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+        /* XXX - as we cannot start a new event loop (using gtk_dialog_run()),
+         * as this will prevent the user from closing the now existing error
+         * message, simply close the dialog (this is the best we can do here). */
+        if (file_save_as_w)
+            window_destroy(file_save_as_w);
+#else
+        gtk_widget_show(file_save_as_w);
+#endif
         break;
     default:
         g_assert_not_reached();
@@ -1719,6 +1758,13 @@ file_color_import_ok_cb(GtkWidget *w, gpointer color_filters) {
        dismiss the alert box popped up for the open error,
        try again. */
     g_free(cf_name);
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    /* XXX - as we cannot start a new event loop (using gtk_dialog_run()),
+     * as this will prevent the user from closing the now existing error
+     * message, simply close the dialog (this is the best we can do here). */
+    if (file_save_as_w)
+      window_destroy(GTK_WIDGET (fs));
+#endif
     return;
   }
 
@@ -1875,8 +1921,14 @@ file_color_export_ok_cb(GtkWidget *w, gpointer filter_list) {
     /* The write failed; don't dismiss the open dialog box,
        just leave it around so that the user can, after they
        dismiss the alert box popped up for the error, try again. */
-
        g_free(cf_name);
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+      /* XXX - as we cannot start a new event loop (using gtk_dialog_run()),
+       * as this will prevent the user from closing the now existing error
+       * message, simply close the dialog (this is the best we can do here). */
+      if (file_save_as_w)
+        window_destroy(GTK_WIDGET (fs));
+#endif
        return;
    }
 
