@@ -1,5 +1,5 @@
 /* flow_graph.c
- * $Id: flow_graph.c 20028 2006-12-01 02:55:45Z sfisher $
+ * $Id: flow_graph.c 24239 2008-02-01 01:07:58Z stig $
  * Allows to display a flow graph of the currently displayed packets
  *
  * Copyright 2004, Ericsson , Spain
@@ -140,6 +140,13 @@ remove_tap_listener_flow_graph(void)
 static void
 flow_graph_on_destroy(GtkObject *object _U_, gpointer user_data _U_)
 {
+#if GTK_MAJOR_VERSION < 2
+	/* Destroy the graph analysis window when we're destroyed.  This is
+	 * handled under GTK2 with a call to gtk_window_set_destroy_with_parent
+	 * in graph_analysis.c */
+	window_destroy(graph_analysis_data->dlg.window);
+#endif
+
 	/* remove_tap_listeners */
 	remove_tap_listener_flow_graph();
 
@@ -341,10 +348,10 @@ static int flow_graph_tcp_add_to_graph(packet_info *pinfo, const struct tcpheade
     bpos = 1 << i;
     if (tcph->th_flags & bpos) {
       if (fpos) {
-        strcpy(&flags[fpos], ", ");
+        strncpy(&flags[fpos], ", ", 64 - fpos - 1);
         fpos += 2;
       }
-      strcpy(&flags[fpos], fstr[i]);
+      strncpy(&flags[fpos], fstr[i], 64 - fpos - 1);
       fpos += 3;
     }
   }
@@ -406,7 +413,7 @@ static void flow_graph_packet_draw(void *prs _U_)
 /****************************************************************************/
 static void
 flow_graph_on_ok                    (GtkButton       *button _U_,
-                                        gpointer         user_data _U_)
+                                        gpointer         user_data)
 {
 
 	if ((have_frame_tap_listener==TRUE)
@@ -455,6 +462,7 @@ flow_graph_on_ok                    (GtkButton       *button _U_,
 		graph_analysis_update(graph_analysis_data);		/* refresh it xxx */
 	}
 	else{
+		graph_analysis_data->dlg.parent_w = user_data;
 		graph_analysis_create(graph_analysis_data);
 	}
 
@@ -617,7 +625,7 @@ static void flow_graph_dlg_create (void)
 	bt_ok = BUTTON_NEW_FROM_STOCK(GTK_STOCK_OK);
 	gtk_container_add(GTK_CONTAINER(hbuttonbox), bt_ok);
 	gtk_tooltips_set_tip (tooltips, bt_ok, "Show the flow graph", NULL);
-	SIGNAL_CONNECT(bt_ok, "clicked", flow_graph_on_ok, NULL);
+	SIGNAL_CONNECT(bt_ok, "clicked", flow_graph_on_ok, flow_graph_dlg_w);
 	gtk_widget_show(bt_ok);
 
 	bt_close = BUTTON_NEW_FROM_STOCK(GTK_STOCK_CLOSE);
