@@ -1,7 +1,7 @@
 /* main.h
  * Global defines, etc.
  *
- * $Id: main.h 22038 2007-06-04 20:15:10Z sfisher $
+ * $Id: main.h 24232 2008-01-31 19:50:38Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -129,14 +129,14 @@ typedef enum {
 /** "bitwise or" this with MATCH_SELECTED_E value for copy to clipboard instead of prepare only */
 #define MATCH_SELECTED_COPY_ONLY    0x200
 
-/** User highlited item in details window and then right clicked and selected the copy option 
+/** User highlited item in details window and then right clicked and selected the copy option
  *
  * @param widget parent widget
  * @param data parent widget
  */
 extern void copy_selected_plist_cb(GtkWidget *w _U_, gpointer data);
 
-/** User requested one of "Apply as Filter" or "Prepare a Filter" functions 
+/** User requested one of "Apply as Filter" or "Prepare a Filter" functions
  *  by menu or context menu of protocol tree.
  *
  * @param widget parent widget
@@ -145,6 +145,17 @@ extern void copy_selected_plist_cb(GtkWidget *w _U_, gpointer data);
  */
 extern void match_selected_ptree_cb(GtkWidget *widget, gpointer data, MATCH_SELECTED_E action);
 
+/** User requested the colorize function
+ *  by menu or context menu of protocol tree.
+ *
+ * @param widget parent widget
+ * @param data parent widget
+ * @param filt_nr  1-10: use filter for color 1-10
+ *                    0: open new colorization rule dialog
+ *                  255: clear filters for color 1-10
+ */
+extern void colorize_selected_ptree_cb(GtkWidget *w, gpointer data, guint8 filt_nr);
+
 /** User requested one of "Apply as Filter" or "Prepare a Filter" functions
  *  by context menu of packet list.
  *
@@ -278,7 +289,7 @@ extern gboolean dfilter_combo_add_recent(gchar *dftext);
 /** Empty out the combobox entry field */
 extern void dfilter_combo_add_empty(void);
 
-/** Write all non empty display filters (until maximum count) 
+/** Write all non empty display filters (until maximum count)
  *  of the combo box GList to the user's recent file.
  *
  * @param rf the recent file
@@ -297,7 +308,7 @@ extern void main_widgets_rearrange(void);
 /** Show or hide the main window widgets, user changed it's preferences. */
 extern void main_widgets_show_or_hide(void);
 
-/** Apply a new filter string. 
+/** Apply a new filter string.
  *  Call cf_filter_packets() and add this filter string to the recent filter list.
  *
  * @param cf the capture file
@@ -322,14 +333,32 @@ extern void dnd_open_file_cmd(gchar *cf_names_freeme);
 /** Update the packets statusbar to the current values. */
 extern void packets_bar_update(void);
 
+/** Update the profile statusbar to the current values. */
+extern void profile_bar_update(void);
+
 #ifdef _WIN32
 /** Win32 only: Create a console. Beware: cannot be closed again. */
 extern void create_console(void);
 #endif
 
-/* Fill in capture options with values from the preferences */
+/** Fill in capture options with values from the preferences */
 extern void prefs_to_capture_opts(void);
 
+/** Change configuration profile */
+extern void change_configuration_profile(const gchar *profile_name);
+
 extern GtkWidget *pkt_scrollw;
 
+/** Push a message referring to the current filter onto the statusbar.
+ *
+ * @param msg The message
+ */
+void
+statusbar_push_filter_msg(const gchar *msg);
+
+/** Pop a message referring to the current filter off the statusbar.
+ */
+void
+statusbar_pop_filter_msg(void);
+
 #endif /* __MAIN_H__ */
