@@ -126,7 +126,7 @@ static void add_to_clist(mcast_stream_info_t* strinfo)
 	gtk_clist_set_row_data(GTK_CLIST(clist), added_row, strinfo);
 
 	/* Update the top label with the number of detected streams */
-	sprintf(label_text,
+	g_snprintf(label_text, 256,
 	        "Detected %d Multicast streams,   Average Bw: %.1f Mbps   Max Bw: %.1f Mbps   Max burst: %d / %dms   Max buffer: %.1f KB",
 	        ++streams_nb, 
 		mcaststream_get_info()->allstreams->average_bw, mcaststream_get_info()->allstreams->element.maxbw, 
@@ -187,10 +187,10 @@ mcaststream_on_filter                    (GtkButton       *button _U_,
 	if (selected_stream_fwd)
 	{
 		if (selected_stream_fwd->src_addr.type==AT_IPv6){
-			strcpy(ip_version,"v6");
+			strncpy(ip_version,"v6",3);
 		}		
 		else{
-			strcpy(ip_version,"");
+			strncpy(ip_version,"",3);
 		}
 		filter_string_fwd = g_strdup_printf(
 			"(ip%s.src==%s && udp.srcport==%u && ip%s.dst==%s && udp.dstport==%u)",
