@@ -1,7 +1,7 @@
 /* proto_draw.c
  * Routines for GTK+ packet display
  *
- * $Id: proto_draw.c 20217 2006-12-26 20:04:54Z ulfl $
+ * $Id: proto_draw.c 23720 2007-12-03 21:57:57Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -90,6 +90,8 @@
 #define E_BYTE_VIEW_TVBUFF_KEY    "byte_view_tvbuff"
 #define E_BYTE_VIEW_START_KEY     "byte_view_start"
 #define E_BYTE_VIEW_END_KEY       "byte_view_end"
+#define E_BYTE_VIEW_APP_START_KEY "byte_view_app_start"
+#define E_BYTE_VIEW_APP_END_KEY   "byte_view_app_end"
 #define E_BYTE_VIEW_ENCODE_KEY    "byte_view_encode"
 
 
@@ -326,6 +328,8 @@ toggle_tree(GtkCTree *ctree, GdkEventKey *event, gpointer user_data _U_)
 				/* number of characters per line;
 				   offset, 2 blanks separating offset
 				   from data dump, data dump */
+#define MAX_LINES       100
+#define MAX_LINES_LEN   (MAX_LINES*MAX_LINE_LEN)
 
 /* Which byte the offset is referring to. Associates
  * whitespace with the preceding digits. */
@@ -819,16 +823,104 @@ savehex_dlg_destroy_cb(void)
         savehex_dlg = NULL;
 }
 
+
+static void copy_hex_all_info(GString* copy_buffer, const guint8* data_p, int data_len, gboolean append_text)
+{
+    const int byte_line_length = 16; /* Print out data for 16 bytes on one line */
+    int i, j;
+    gboolean end_of_line = TRUE; /* Initial state is end of line */
+    int byte_line_part_length;
+
+    GString* hex_str;
+    GString* char_str;
+
+    /* Write hex data for a line, then ascii data, then concatenate and add to buffer */
+    hex_str = g_string_new("");
+    char_str= g_string_new("");
+
+    i = 0;
+	while (i<data_len){
+        if(end_of_line) {
+            g_string_sprintfa(hex_str,"%04x  ",i); /* Offset - note that we _append_ here */
+        }
+
+        g_string_sprintfa(hex_str," %02x",*data_p);
+        if(append_text) {
+            g_string_sprintfa(char_str,"%c",isprint(*data_p) ? *data_p : '.');
+        }
+
+        ++data_p;
+
+        /* Look ahead to see if this is the end of the data */
+        byte_line_part_length = (++i) % byte_line_length;
+        if(i == data_len){
+            /* End of data - need to fill in spaces in hex string and then do "end of line".
+             * 
+             */
+            for(j = 0; append_text && (j < (byte_line_length - byte_line_part_length)); ++j) {
+                g_string_append(hex_str,"   "); /* Three spaces for each missing byte */
+            }
+            end_of_line = TRUE;
+        } else {
+            end_of_line = (byte_line_part_length == 0 ? TRUE : FALSE);
+        }
+        
+
+        if (end_of_line){
+            /* End of line */
+            g_string_append(copy_buffer, hex_str->str);
+            if(append_text) {
+                /* Two spaces between hex and text */
+                g_string_append_c(copy_buffer, ' ');
+                g_string_append_c(copy_buffer, ' ');
+                g_string_append(copy_buffer, char_str->str);
+            }
+            /* Setup ready for next line */
+            g_string_assign(char_str,"");
+            g_string_assign(hex_str, "\n");
+        }
+	}
+
+	g_string_free(hex_str, TRUE);
+	g_string_free(char_str, TRUE);
+}
+
+static 
+int copy_hex_bytes_text_only(GString* copy_buffer, const guint8* data_p, int data_len _U_)
+{
+
+    gchar to_append;
+
+    if(isprint(*data_p)) {
+        to_append = *data_p;
+    } else if(*data_p==0x0a) {
+        to_append = '\n'; /* Copied from old copy_hex_cb function; not sure why this is needed - portablity?*/
+    } else {
+        return 1; /* Just ignore non-printable bytes */
+    }
+    g_string_append_c(copy_buffer,to_append);
+    return 1;
+}
+
+static 
+int copy_hex_bytes_hex(GString* copy_buffer, const guint8* data_p, int data_len _U_)
+{
+    g_string_sprintfa(copy_buffer, "%02x", *data_p);
+    return 1;
+}
+
 void
-copy_hex_cb(GtkWidget * w _U_, gpointer data _U_, int data_type)
+copy_hex_cb(GtkWidget * w _U_, gpointer data _U_, copy_data_type data_type)
 {
 	GtkWidget *bv;
-	int len;
-	int i=0;
-	const guint8 *data_p = NULL;
-	GString *ASCII_representation = g_string_new("");
-	GString *byte_str = g_string_new("");
-	GString *text_str = g_string_new("");
+
+    guint len = 0;
+    int bytes_consumed = 0;
+    int flags;
+
+    const guint8* data_p;
+
+	GString* copy_buffer = g_string_new(""); /* String to copy to clipboard */
 
 	bv = get_notebook_bv_ptr(byte_nb_ptr);
 	if (bv == NULL) {
@@ -840,50 +932,68 @@ copy_hex_cb(GtkWidget * w _U_, gpointer data _U_, int data_type)
 	data_p = get_byte_view_data_and_length(bv, &len);
 	g_assert(data_p != NULL);
 
-    g_string_sprintfa(byte_str,"%04x  ",i); /* Offset 0000 */
-	for (i=0; i<len; i++){
-        if (data_type==1) {
-            if (isprint(*data_p)) {
-                g_string_sprintfa(ASCII_representation,"%c", *data_p);
-            }
-            else
-            {
-                if (*data_p==0x0a) {
-                    g_string_sprintfa(ASCII_representation,"\n");
-                }
-            }
-        }
-        else
-        {
-            g_string_sprintfa(ASCII_representation,"%c",isprint(*data_p) ? *data_p : '.');
-        }
-        g_string_sprintfa(byte_str," %02x",*data_p++);
-        if ((i+1)%16==0 && i!=0){
-            g_string_sprintfa(byte_str,"  %s\n%04x  ",ASCII_representation->str,i+1);
-            g_string_sprintfa(text_str,"%s",ASCII_representation->str);
+    flags = data_type & CD_FLAGSMASK;
+    data_type = data_type & CD_TYPEMASK;
 
-            g_string_assign (ASCII_representation,"");
-        }
-	}
+    if(flags & CD_FLAGS_SELECTEDONLY) {
+        int start, end;
+        
+        /* Get the start and end of the highlighted bytes.
+         * XXX The keys appear to be REVERSED start <-> end throughout this file!
+         * Should this be fixed? There is one exception - packet_hex_reprint,
+         * so can't just change it round without functional change.
+         */
+	    end = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_START_KEY));
+	    start = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_END_KEY));
 
-	if(ASCII_representation->len){
-	  for (i=ASCII_representation->len; i<16; i++){
-	    g_string_sprintfa(byte_str,"   ");
-	  }
-	  g_string_sprintfa(byte_str,"  %s\n",ASCII_representation->str);
-	  g_string_sprintfa(text_str,"%s",ASCII_representation->str);
-	}
-	/* Now that we have the byte data, copy it into the default clipboard */
-    if (data_type==1) {
-        copy_to_clipboard(text_str);
+	if(start >= 0 && end > start && (end - start <= (int)len)) {
+            len = end - start;
+            data_p += start;
+        }
     }
-    else
-    {
-        copy_to_clipboard(byte_str);
+
+    switch(data_type) {
+    case(CD_ALLINFO):
+        /* This is too different from other text formats - handle separately */
+        copy_hex_all_info(copy_buffer, data_p, len, TRUE);
+        break;
+    case(CD_HEXCOLUMNS):
+        /* This could be done incrementally, but it is easier to mingle with the code for CD_ALLINFO */
+        copy_hex_all_info(copy_buffer, data_p, len, FALSE);
+        break;
+#if (GTK_MAJOR_VERSION >= 2)
+    case(CD_BINARY):
+        /* Completely different logic to text copies - leave copy buffer alone */
+        copy_binary_to_clipboard(data_p,len);
+        break;
+#endif
+    default:
+        /* Incrementally write to text buffer in various formats */
+	    while (len > 0){
+            switch(data_type) {
+            case (CD_TEXTONLY):
+                bytes_consumed = copy_hex_bytes_text_only(copy_buffer, data_p, len);
+                break;
+            case (CD_HEX):
+                bytes_consumed = copy_hex_bytes_hex(copy_buffer, data_p, len);
+                break;
+            default:
+                g_assert_not_reached();
+                break;
+            }
+
+            g_assert(bytes_consumed>0);
+            data_p += bytes_consumed;
+            len -= bytes_consumed;
+        }
+        break;
     }
-	g_string_free(byte_str, TRUE);                       /* Free the memory */
-	g_string_free(text_str, TRUE);                       /* Free the memory */
-	g_string_free(ASCII_representation, TRUE);           /* Free the memory */
+    
+    if(copy_buffer->len > 0) {
+        copy_to_clipboard(copy_buffer);
+    }
+
+	g_string_free(copy_buffer, TRUE);  
 }
 
 /* save the current highlighted hex data */
@@ -891,7 +1001,8 @@ static void
 savehex_save_clicked_cb(GtkWidget * w _U_, gpointer data _U_)
 {
         GtkWidget *bv;
-	int fd, start, end, len;
+	int fd, start, end;
+	guint len;
 	const guint8 *data_p = NULL;
 	const char *file = NULL;
 
@@ -949,7 +1060,8 @@ savehex_save_clicked_cb(GtkWidget * w _U_, gpointer data _U_)
 /* Launch the dialog box to put up the file selection box etc */
 void savehex_cb(GtkWidget * w _U_, gpointer data _U_)
 {
-	int start, end, len;
+	int start, end;
+	guint len;
 	const guint8 *data_p = NULL;
         gchar *label;
 
@@ -1043,10 +1155,10 @@ void savehex_cb(GtkWidget * w _U_, gpointer data _U_)
  */
 static void
 packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
-			int bend, int encoding)
+			int bend, int astart, int aend, int encoding)
 {
   int            i = 0, j, k, cur;
-  guchar         line[MAX_LINE_LEN + 1];
+  guchar         line[MAX_LINES_LEN + 1];
   static guchar  hexchars[16] = {
       '0', '1', '2', '3', '4', '5', '6', '7',
       '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
@@ -1062,8 +1174,6 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
   GtkTextBuffer *buf = gtk_text_view_get_buffer(bv_text_view);
   GtkTextIter    iter;
   const char    *revstyle;
-  gchar         *convline;
-  gsize          newsize;
   GtkTextMark   *mark = NULL;
 #endif
 
@@ -1090,6 +1200,8 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
 #else
   gtk_text_buffer_set_text(buf, "", 0);
   gtk_text_buffer_get_start_iter(buf, &iter);
+  g_object_ref(buf);  
+  gtk_text_view_set_buffer( bv_text_view, NULL);
 #endif
 
   /*
@@ -1122,6 +1234,7 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
   progbar_stop_flag = FALSE;
   g_get_current_time(&progbar_start_time);
 
+  cur = 0;
   while (i < len) {
     /* Create the progress bar if necessary.
        We check on every iteration of the loop, so that it takes no
@@ -1165,7 +1278,6 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
 
     /* Print the line number */
     j = use_digits;
-    cur = 0;
     do {
       j--;
       c = (i >> (j*4)) & 0xF;
@@ -1173,14 +1285,13 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
     } while (j != 0);
     line[cur++] = ' ';
     line[cur++] = ' ';
-    line[cur] = '\0';
 
     /* Display with inverse video ? */
 #if GTK_MAJOR_VERSION < 2
     if (prefs.gui_hex_dump_highlight_style) {
-      gtk_text_insert(bv_text, user_font_get_regular(), &BLACK, &WHITE, line, -1);
+      gtk_text_insert(bv_text, user_font_get_regular(), &BLACK, &WHITE, line, cur);
       /* Do we start in reverse? */
-      reverse = i >= bstart && i < bend;
+      reverse = (i >= bstart && i < bend) || (i >= astart && i < aend);
       fg = reverse ? &WHITE : &BLACK;
       bg = reverse ? &BLACK : &WHITE;
       j   = i;
@@ -1195,7 +1306,7 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
 	  line[cur++] = ' '; line[cur++] = ' ';
 	}
 	i++;
-	newreverse = i >= bstart && i < bend;
+	newreverse = (i >= bstart && i < bend) || (i >= astart && i < aend);
 	/* Have we gone from reverse to plain? */
 	if (reverse && (reverse != newreverse)) {
 	  gtk_text_insert(bv_text, user_font_get_regular(), fg, bg, line, cur);
@@ -1231,7 +1342,7 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
       /* Print the ASCII bit */
       i = j;
       /* Do we start in reverse? */
-      reverse = i >= bstart && i < bend;
+      reverse = (i >= bstart && i < bend) || (i >= astart && i < aend);
       fg = reverse ? &WHITE : &BLACK;
       bg = reverse ? &BLACK : &WHITE;
       while (i < k) {
@@ -1250,7 +1361,7 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
 	  line[cur++] = ' ';
 	}
 	i++;
-	newreverse = i >= bstart && i < bend;
+	newreverse = (i >= bstart && i < bend) || (i >= astart && i < aend);
 	/* Have we gone from reverse to plain? */
 	if (reverse && (reverse != newreverse)) {
 	  gtk_text_insert(bv_text, user_font_get_regular(), fg, bg, line, cur);
@@ -1274,16 +1385,14 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
 	reverse = newreverse;
       }
       /* Print remaining part of line */
+      line[cur++] = '\n';
       gtk_text_insert(bv_text, user_font_get_regular(), fg, bg, line, cur);
       cur = 0;
-      line[cur++] = '\n';
-      line[cur]   = '\0';
-      gtk_text_insert(bv_text, user_font_get_regular(), &BLACK, &WHITE, line, -1);
     }
     else {
-      gtk_text_insert(bv_text, user_font_get_regular(), NULL, NULL, line, -1);
+      gtk_text_insert(bv_text, user_font_get_regular(), NULL, NULL, line, cur);
       /* Do we start in bold? */
-      cur_font = (i >= bstart && i < bend) ? user_font_get_bold() : user_font_get_regular();
+      cur_font = ((i >= bstart && i < bend) || (i >= astart && i < aend)) ? user_font_get_bold() : user_font_get_regular();
       j   = i;
       k   = i + BYTE_VIEW_WIDTH;
       cur = 0;
@@ -1300,7 +1409,7 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
 	/* insert a space every BYTE_VIEW_SEP bytes */
 	if( ( i % BYTE_VIEW_SEP ) == 0 ) line[cur++] = ' ';
 	/* Did we cross a bold/plain boundary? */
-	new_font = (i >= bstart && i < bend) ? user_font_get_bold() : user_font_get_regular();
+	new_font = ((i >= bstart && i < bend) || (i >= astart && i < aend)) ? user_font_get_bold() : user_font_get_regular();
 	if (cur_font != new_font) {
 	  gtk_text_insert(bv_text, cur_font, NULL, NULL, line, cur);
 	  cur_font = new_font;
@@ -1313,7 +1422,7 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
       cur = 0;
       i = j;
       /* Print the ASCII bit */
-      cur_font = (i >= bstart && i < bend) ? user_font_get_bold() : user_font_get_regular();
+      cur_font = ((i >= bstart && i < bend) || (i >= astart && i < aend)) ? user_font_get_bold() : user_font_get_regular();
       while (i < k) {
 	if (i < len) {
 	  if (encoding == CHAR_ASCII) {
@@ -1333,7 +1442,7 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
 	/* insert a space every BYTE_VIEW_SEP bytes */
 	if( ( i % BYTE_VIEW_SEP ) == 0 ) line[cur++] = ' ';
 	/* Did we cross a bold/plain boundary? */
-	new_font = (i >= bstart && i < bend) ? user_font_get_bold() : user_font_get_regular();
+	new_font = ((i >= bstart && i < bend) || (i >= astart && i < aend)) ? user_font_get_bold() : user_font_get_regular();
 	if (cur_font != new_font) {
 	  gtk_text_insert(bv_text, cur_font, NULL, NULL, line, cur);
 	  cur_font = new_font;
@@ -1341,8 +1450,8 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
 	}
       }
       line[cur++] = '\n';
-      line[cur]   = '\0';
-      gtk_text_insert(bv_text, cur_font, NULL, NULL, line, -1);
+      gtk_text_insert(bv_text, cur_font, NULL, NULL, line, cur);
+      cur = 0;
     }
 #else
     if (prefs.gui_hex_dump_highlight_style)
@@ -1350,13 +1459,15 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
     else
       revstyle = "bold";
 
-    gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, -1, "plain",
-                                             NULL);
     /* Do we start in reverse? */
-    reverse = i >= bstart && i < bend;
+    reverse = (i >= bstart && i < bend) || (i >= astart && i < aend);
     j   = i;
     k   = i + BYTE_VIEW_WIDTH;
-    cur = 0;
+    if (reverse) {
+      gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
+					     "plain", NULL);
+      cur = 0;
+    }
     /* Print the hex bit */
     while (i < k) {
       if (i < len) {
@@ -1366,10 +1477,10 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
         line[cur++] = ' '; line[cur++] = ' ';
       }
       i++;
-      newreverse = i >= bstart && i < bend;
+      newreverse = (i >= bstart && i < bend) || (i >= astart && i < aend);
       /* Have we gone from reverse to plain? */
       if (reverse && (reverse != newreverse)) {
-        gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
+	gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
                                                  revstyle, NULL);
         cur = 0;
       }
@@ -1383,28 +1494,33 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
       }
       /* Have we gone from plain to reversed? */
       if (!reverse && (reverse != newreverse)) {
-        gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
+	gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
                                                  "plain", NULL);
         mark = gtk_text_buffer_create_mark(buf, NULL, &iter, TRUE);
         cur = 0;
       }
       reverse = newreverse;
     }
-    /* Print remaining part of line */
-    gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
-                                             reverse ? revstyle : "plain",
-                                             NULL);
-    cur = 0;
+    if (reverse) {
+      /* Print remaining part of line */
+      gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
+                                             revstyle, NULL);
+      cur = 0;
+    }
+
     /* Print some space at the end of the line */
     line[cur++] = ' '; line[cur++] = ' '; line[cur++] = ' ';
-    gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
-                                             "plain", NULL);
-    cur = 0;
 
     /* Print the ASCII bit */
     i = j;
     /* Do we start in reverse? */
-    reverse = i >= bstart && i < bend;
+    reverse = (i >= bstart && i < bend) || (i >= astart && i < aend);
+    if (reverse) {
+      gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
+                                             "plain",NULL);
+      cur = 0;
+    }
+
     while (i < k) {
       if (i < len) {
         if (encoding == CHAR_ASCII) {
@@ -1421,13 +1537,12 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
         line[cur++] = ' ';
       }
       i++;
-      newreverse = i >= bstart && i < bend;
+      newreverse = (i >= bstart && i < bend) || (i >= astart && i < aend);
       /* Have we gone from reverse to plain? */
       if (reverse && (reverse != newreverse)) {
-        convline = g_locale_to_utf8(line, cur, NULL, &newsize, NULL);
-        gtk_text_buffer_insert_with_tags_by_name(buf, &iter, convline, newsize,
+        gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
                                                  revstyle, NULL);
-        g_free( (gpointer) convline);
+        
         cur = 0;
       }
       if (i < k) {
@@ -1438,24 +1553,24 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
       }
       /* Have we gone from plain to reversed? */
       if (!reverse && (reverse != newreverse)) {
-        convline = g_locale_to_utf8(line, cur, NULL, &newsize, NULL);
-        gtk_text_buffer_insert_with_tags_by_name(buf, &iter, convline, newsize,
+        gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
                                                  "plain", NULL);
-        g_free( (gpointer) convline);
         cur = 0;
       }
       reverse = newreverse;
     }
     /* Print remaining part of line */
-    convline = g_locale_to_utf8(line, cur, NULL, &newsize, NULL);
-    gtk_text_buffer_insert_with_tags_by_name(buf, &iter, convline, newsize,
-                                             reverse ? revstyle : "plain",
-                                             NULL);
-    g_free( (gpointer) convline);
-    cur = 0;
+    if (reverse) {
+        gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
+                                             revstyle, NULL);
+        cur = 0;
+    }
     line[cur++] = '\n';
-    gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
+    if (cur >= (MAX_LINES_LEN - MAX_LINE_LEN)) {
+        gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
                                              "plain", NULL);
+        cur = 0;
+    }
 #endif
   }
 
@@ -1478,10 +1593,17 @@ packet_hex_print_common(GtkWidget *bv, const guint8 *pd, int len, int bstart,
     gtk_adjustment_set_value(bv_text->vadj, scrollval);
   }
 #else
+  if (cur) {
+        gtk_text_buffer_insert_with_tags_by_name(buf, &iter, line, cur,
+                                             "plain", NULL);
+  }
+  gtk_text_view_set_buffer( bv_text_view, buf);
+  
   if (mark) {
     gtk_text_view_scroll_to_mark(bv_text_view, mark, 0.0, TRUE, 1.0, 0.0);
     gtk_text_buffer_delete_mark(buf, mark);
   }
+  g_object_unref(buf);  
 #endif
 }
 
@@ -1492,27 +1614,32 @@ packet_hex_print(GtkWidget *bv, const guint8 *pd, frame_data *fd,
   /* do the initial printing and save the information needed 	*/
   /* to redraw the display if preferences change.		*/
 
-  int bstart, bend = -1, blen;
+  int bstart = -1, bend = -1, blen = -1;
+  int astart = -1, aend = -1, alen = -1;
 
   if (finfo != NULL) {
     bstart = finfo->start;
     blen = finfo->length;
-  } else {
-    bstart = -1;
-    blen = -1;
+    astart = finfo->appendix_start;
+    alen = finfo->appendix_length;
   }
   if (bstart >= 0 && blen >= 0) {
     bend = bstart + blen;
   }
+  if (astart >= 0 && alen >= 0) {
+    aend = astart + alen;
+  }
 
   /* save the information needed to redraw the text */
   /* should we save the fd & finfo pointers instead ?? */
   OBJECT_SET_DATA(bv, E_BYTE_VIEW_START_KEY, GINT_TO_POINTER(bend));
   OBJECT_SET_DATA(bv, E_BYTE_VIEW_END_KEY, GINT_TO_POINTER(bstart));
+  OBJECT_SET_DATA(bv, E_BYTE_VIEW_APP_START_KEY, GINT_TO_POINTER(aend));
+  OBJECT_SET_DATA(bv, E_BYTE_VIEW_APP_END_KEY, GINT_TO_POINTER(astart));
   OBJECT_SET_DATA(bv, E_BYTE_VIEW_ENCODE_KEY,
-                  GINT_TO_POINTER(fd->flags.encoding));
+                  GUINT_TO_POINTER((guint)fd->flags.encoding));
 
-  packet_hex_print_common(bv, pd, len, bstart, bend, fd->flags.encoding);
+  packet_hex_print_common(bv, pd, len, bstart, bend, astart, aend, fd->flags.encoding);
 }
 
 /*
@@ -1523,16 +1650,19 @@ void
 packet_hex_reprint(GtkWidget *bv)
 {
   int start, end, encoding;
+  int astart, aend;
   const guint8 *data;
-  guint len;
+  guint len = 0;
 
   start = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_START_KEY));
   end = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_END_KEY));
+  astart = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_APP_START_KEY));
+  aend = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_APP_END_KEY));
   data = get_byte_view_data_and_length(bv, &len);
   g_assert(data != NULL);
   encoding = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_ENCODE_KEY));
 
-  packet_hex_print_common(bv, data, len, start, end, encoding);
+  packet_hex_print_common(bv, data, len, start, end, astart, aend, encoding);
 }
 
 /* List of all protocol tree widgets, so we can globally set the selection
@@ -1882,12 +2012,12 @@ tree_view_follow_link(field_info   *fi)
     gchar *url;
 
     if(fi->hfinfo->type == FT_FRAMENUM) {
-        cf_goto_frame(&cfile, fi->value.value.integer);
+        cf_goto_frame(&cfile, fi->value.value.uinteger);
     }
     if(FI_GET_FLAG(fi, FI_URL) && IS_FT_STRING(fi->hfinfo->type)) {
-      url = g_strndup(tvb_get_ptr(fi->ds_tvb, fi->start, fi->length), fi->length);
-      browser_open_url(url);
-      g_free(url);
+        url = g_strndup(tvb_get_ptr(fi->ds_tvb, fi->start, fi->length), fi->length);
+        browser_open_url(url);
+        g_free(url);
     }
 }
 
@@ -1916,13 +2046,15 @@ tree_view_select(GtkWidget *widget, GdkEventButton *event)
             node = gtk_ctree_node_nth(ctree, row);
             g_assert(node);
 
-            gtk_ctree_select(ctree, node);
-
             /* if that's a doubleclick, try to follow the link */
             if(event->type == GDK_2BUTTON_PRESS) {
                 fi = gtk_ctree_node_get_row_data(ctree, node);
                 tree_view_follow_link(fi);
             }
+            else if (((GdkEventButton *)event)->button != 1) {
+                /* if button == 1 gtk_ctree_select is already (or will be) called by the widget */
+                gtk_ctree_select(ctree, node);
+            }
         } else {
             return FALSE;
         }
@@ -1936,7 +2068,6 @@ tree_view_select(GtkWidget *widget, GdkEventButton *event)
                                           &path, NULL, NULL, NULL))
         {
             sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
-            gtk_tree_selection_select_path(sel, path);
 
             /* if that's a doubleclick, try to follow the link */
             if(event->type == GDK_2BUTTON_PRESS) {
@@ -1949,6 +2080,10 @@ tree_view_select(GtkWidget *widget, GdkEventButton *event)
                     tree_view_follow_link(fi);
                 }
             }
+            else if (((GdkEventButton *)event)->button != 1) {
+                /* if button == 1 gtk_tree_selection_select_path is already (or will be) called by the widget */
+                gtk_tree_selection_select_path(sel, path);
+            }
         } else {
             return FALSE;
         }
