@@ -1,7 +1,7 @@
 /* 
  * Copyright 2004, Irene Ruengeler <i.ruengeler [AT] fh-muenster.de>
  *
- * $Id: sctp_stat.c 20630 2007-01-30 19:30:25Z gerald $
+ * $Id: sctp_stat.c 24239 2008-02-01 01:07:58Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -150,8 +150,8 @@ static void tsn_free(gpointer data, gpointer user_data _U_)
 	}
 }
 
-static void
-reset(void *arg)
+
+static void reset(void *arg)
 {
 	sctp_allassocs_info_t *tapdata = arg;
 	GList* list;
@@ -264,7 +264,7 @@ static sctp_assoc_info_t *calc_checksum(struct _sctp_info *check_data, sctp_asso
 	{
 		if ((float)(data->n_adler32_correct*1.0/data->n_adler32_calculated) > 0.5)
 		{
-			strcpy(data->checksum_type,"ADLER32");
+			strncpy(data->checksum_type,"ADLER32",8);
 			data->n_checksum_errors=(data->n_adler32_calculated-data->n_adler32_correct);
 			ok = TRUE;
 		}
@@ -274,7 +274,7 @@ static sctp_assoc_info_t *calc_checksum(struct _sctp_info *check_data, sctp_asso
 	{
 		if ((float)(data->n_crc32c_correct*1.0/data->n_crc32c_calculated) > 0.5)
 		{
-			strcpy(data->checksum_type,"CRC32C");
+			strncpy(data->checksum_type,"CRC32C",8);
 			data->n_checksum_errors=data->n_crc32c_calculated-data->n_crc32c_correct;
 			ok = TRUE;
 		}
@@ -282,7 +282,7 @@ static sctp_assoc_info_t *calc_checksum(struct _sctp_info *check_data, sctp_asso
 
 	if (!ok)
 	{
-		strcpy(data->checksum_type,"UNKNOWN");
+		strncpy(data->checksum_type,"UNKNOWN",8);
 		data->n_checksum_errors=0;
 	}
 
@@ -307,7 +307,7 @@ static gint sctp_assoc_vtag_cmp(gconstpointer aa, gconstpointer bb)
 	    (a->port2 == b->port2) &&
 	    (a->verification_tag1 == b->verification_tag1) && a->verification_tag1==0 && a->initiate_tag != 0 &&
 	    (a->initiate_tag != b->initiate_tag ))
-		return(ASSOC_NOT_FOUND);   //two INITs that belong to different assocs
+		return(ASSOC_NOT_FOUND);   /* two INITs that belong to different assocs */
 
 	/* assoc known*/
 	if ((a->port1 == b->port1) &&
@@ -317,6 +317,13 @@ static gint sctp_assoc_vtag_cmp(gconstpointer aa, gconstpointer bb)
 	     (b->verification_tag2 != 0))))
 		return(FORWARD_STREAM);
 
+	/* ABORT, vtag reflected */
+	if ((a->port1 == b->port1) &&
+	    (a->port2 == b->port2) &&
+	    (a->verification_tag2 == b->verification_tag2) &&	
+	    (a->verification_tag1 == 0 && b->verification_tag1 != 0))
+		return(FORWARD_STREAM);
+
 	if ((a->port1 == b->port2) &&
 	    (a->port2 == b->port1) &&
 	    (a->verification_tag1 == b->verification_tag2) &&
@@ -329,6 +336,13 @@ static gint sctp_assoc_vtag_cmp(gconstpointer aa, gconstpointer bb)
 	    (a->verification_tag2 != 0))
 		return(BACKWARD_STREAM);
 
+	/* ABORT, vtag reflected */
+	if ((a->port1 == b->port2) &&
+	    (a->port2 == b->port1) &&
+	    (a->verification_tag2 == b->verification_tag1) &&	
+	    (a->verification_tag1 == 0 && b->verification_tag2 != 0))
+		return(BACKWARD_STREAM);
+
 	/*forward stream verifivation tag can be added*/
 	if ((a->port1 == b->port1) &&
 	    (a->port2 == b->port2) &&
@@ -354,236 +368,6 @@ static gint sctp_assoc_vtag_cmp(gconstpointer aa, gconstpointer bb)
 	return(ASSOC_NOT_FOUND);
 }
 
-/*
-static gint sctp_assoc_address_cmp(gconstpointer aa, gconstpointer bb)
-{
-	GList *srclist, *dstlist;
-	const struct _sctp_tmp_info* a = aa;
-	const struct _sctp_assoc_info* b = bb;
-	address *srcstore=NULL;
-	address *dststore=NULL;
-	address *src=NULL;
-	address *dst=NULL;
-	gboolean src_v4=FALSE;
-	gboolean src_v6=FALSE;
-	gboolean dst_v4=FALSE;
-	gboolean dst_v6=FALSE;
-	guint8* addr;
-
-	src = g_malloc(sizeof(address));
-	if (a->src.type == AT_IPv4)
-	{
-		src->type = AT_IPv4;
-		src->len  = 4;
-		src_v4    = TRUE;
-	}
-	else if (a->src.type==AT_IPv6)
-	{
-		src->type = AT_IPv6;
-		src->len  = 16;
-		src_v6    = TRUE;
-	}
-	addr = g_malloc(src->len);
-	memcpy(addr, a->src.data, src->len);
-	src->data = addr;
-
-	dst = g_malloc(sizeof(address));
-	if (a->dst.type == AT_IPv4)
-	{
-		dst->type = AT_IPv4;
-		dst->len  = 4;
-		dst_v4    = TRUE;
-	}
-	else if (a->dst.type==AT_IPv6)
-	{
-		dst->type = AT_IPv6;
-		dst->len  = 16;
-		dst_v6    = TRUE;
-	}
-	addr = g_malloc(dst->len);
-	memcpy(addr, a->dst.data, dst->len);
-	dst->data = addr;
-
-	srclist = g_list_first(b->addr1);
-	while (srclist)
-	{
-		srcstore = (address *) (srclist->data);
-		if (srcstore->type==AT_IPv4 && src_v4==TRUE)
-		{
-			if (*src->data==*srcstore->data && a->port1 == b->port1)
-			{
-				dstlist = g_list_first(b->addr2);
-				while (dstlist)
-				{
-						dststore = (address *) (dstlist->data);
-						if ((dststore->type==AT_IPv4 && dst_v4==TRUE) ||(dststore->type==AT_IPv6 && dst_v6==TRUE) )
-						{
-							if (*dst->data==*dststore->data && a->port2 == b->port2)
-							{
-								if ((a->verification_tag1 !=0)&& (b->verification_tag1 == 0)&& (b->verification_tag2 !=0))
-									return ADDRESS_FORWARD_ADD_FORWARD_VTAG;
-								else
-									return ADDRESS_FORWARD_STREAM;
-							}
-							else
-								dstlist=g_list_next(dstlist);
-						}
-						else
-							dstlist=g_list_next(dstlist);
-				}
-				srclist=g_list_next(srclist);
-			}
-			else
-				srclist=g_list_next(srclist);
-		}
-		else if (srcstore->type==AT_IPv6 && src_v6==TRUE)
-		{
-			if (*src->data == *srcstore->data  && a->port1 == b->port1)
-			{
-				dstlist = g_list_first(b->addr2);
-				while (dstlist)
-				{
-					dststore = (address *) (dstlist->data);
-					if ((dststore->type==AT_IPv4 && dst_v4==TRUE) || (dststore->type==AT_IPv6 && dst_v6==TRUE))
-					{
-						if (*dst->data==*dststore->data && a->port2 == b->port2)
-						{
-							if ((a->verification_tag1 !=0)&& (b->verification_tag1 == 0)&& (b->verification_tag2 !=0))
-								return ADDRESS_FORWARD_ADD_FORWARD_VTAG;
-							else
-								return ADDRESS_FORWARD_STREAM;
-						}
-						else
-							dstlist=g_list_next(dstlist);
-					}
-					else
-						dstlist=g_list_next(dstlist);
-				}
-				srclist=g_list_next(srclist);
-			}
-			else
-				srclist=g_list_next(srclist);
-		}
-		else
-			srclist=g_list_next(srclist);
-	}
-
-	g_free(src);
-	g_free(dst);
-
-	src = g_malloc(sizeof(address));
-	if (a->dst.type == AT_IPv4)
-	{
-		src->type = AT_IPv4;
-		src->len  = 4;
-		src_v4    = TRUE;
-	}
-	else if (a->dst.type==AT_IPv6)
-	{
-		src->type = AT_IPv6;
-		src->len  = 16;
-		src_v6    = TRUE;
-	}
-	addr = g_malloc(src->len);
-	memcpy(addr, a->dst.data, src->len);
-	src->data = addr;
-	
-	dst = g_malloc(sizeof(address));
-	if (a->src.type == AT_IPv4)
-	{
-		dst->type = AT_IPv4;
-		dst->len  = 4;
-		dst_v4    = TRUE;
-	}
-	else if (a->src.type==AT_IPv6)
-	{
-		dst->type = AT_IPv6;
-		dst->len  = 16;
-		dst_v6    = TRUE;
-	}
-	addr = g_malloc(dst->len);
-	memcpy(addr, a->src.data, dst->len);
-	dst->data = addr;
-	
-	srclist = g_list_first(b->addr1);
-	while (srclist)
-	{
-		srcstore = (address *) (srclist->data);
-		if (srcstore->type==AT_IPv4 && src_v4==TRUE)
-		{
-			if (*src->data==*srcstore->data && a->port2 == b->port1)
-			{
-				dstlist = g_list_first(b->addr2);
-				while (dstlist)
-				{
-						dststore = (address *) (dstlist->data);
-						if ((dststore->type==AT_IPv4 && src_v4==TRUE) || (dststore->type==AT_IPv6 && src_v6==TRUE))
-						{
-							if (*dst->data==*dststore->data && a->port1 == b->port2)
-							{
-								if ((a->verification_tag1 ==b->verification_tag2)&& (b->verification_tag1 == 0))
-									return ADDRESS_BACKWARD_ADD_FORWARD_VTAG;
-								else if ((a->verification_tag1 !=0)	&& (b->verification_tag1 != 0)&& (b->verification_tag2 == 0))
-									return ADDRESS_BACKWARD_ADD_BACKWARD_VTAG;
-								else
-									return ADDRESS_BACKWARD_STREAM;
-							}
-							else
-								dstlist=g_list_next(dstlist);
-						}
-						else
-							dstlist=g_list_next(dstlist);
-				}
-				srclist=g_list_next(srclist);
-			}
-			else
-				srclist=g_list_next(srclist);
-		}
-		else if (srcstore->type==AT_IPv6 && src_v6==TRUE)
-		{
-			if (*src->data == *srcstore->data && a->port2 == b->port1)
-			{
-				dstlist = g_list_first(b->addr2);
-				while (dstlist)
-				{
-					dststore = (address *) (dstlist->data);
-					if ((dststore->type==AT_IPv4 && src_v4==TRUE) || (dststore->type==AT_IPv6 && src_v6==TRUE))
-					{
-						if (*dst->data==*dststore->data && a->port1 == b->port2)
-						{
-								if ((a->verification_tag1 ==b->verification_tag2)&& (b->verification_tag1 == 0))
-									return ADDRESS_BACKWARD_ADD_FORWARD_VTAG;
-								else if ((a->verification_tag1 !=0)	&& (b->verification_tag1 != 0)&& (b->verification_tag2 == 0))
-									return ADDRESS_BACKWARD_ADD_BACKWARD_VTAG;
-								else
-									return ADDRESS_BACKWARD_STREAM;
-						}
-						else
-							dstlist=g_list_next(dstlist);
-					}
-					else
-						dstlist=g_list_next(dstlist);
-				}
-				srclist=g_list_next(srclist);
-			}
-			else
-				srclist=g_list_next(srclist);
-		}
-		else
-			srclist=g_list_next(srclist);
-	}
-
-
-	g_free(src);
-	g_free(dst);
-	return ASSOC_NOT_FOUND;
-}
-*/
-
-
-
-
-
 static sctp_assoc_info_t * find_assoc(sctp_tmp_info_t * needle)
 {
 	sctp_allassocs_info_t *assoc_info;
@@ -679,7 +463,7 @@ static sctp_assoc_info_t * add_chunk_count(address * vadd, sctp_assoc_info_t * i
 		if (ch->direction == direction)
 		{
 			v = (address *) (ch->addr);
-			if (*(vadd->data)==*(v->data))
+			if (ADDRESSES_EQUAL(vadd, v))
 			{
 				ch->addr_count[type]++;
 				return info;
@@ -721,41 +505,18 @@ static sctp_assoc_info_t * add_address(address * vadd, sctp_assoc_info_t *info,
 	while (list)
 	{
 		v = (address *) (list->data);
-		if (v->type == AT_IPv4 && vadd->type == AT_IPv4)
-		{
-			if (*vadd->data!=*v->data)
-			{
-				list = g_list_next(list);
-			}
-			else
-			{
-				g_free(vadd);
-				return info;
-			}
-		}
-		else if (v->type == AT_IPv6 && vadd->type == AT_IPv6)
-		{
-			if (strcmp(ip6_to_str((const struct e_in6_addr *)(vadd->data)), ip6_to_str((const struct e_in6_addr *)v->data)))
-			{
-				list = g_list_next(list);
-			}
-			else
-			{
-				g_free(vadd);
-				return info;
-			}
-		}
-		else
-		{
-			list = g_list_next(list);
+		if (ADDRESSES_EQUAL(vadd, v)) {
+			g_free(vadd);
+			return info;
 		}
+		list = g_list_next(list);
 	}
 
 	if (direction == 1)
 		info->addr1 = g_list_append(info->addr1, vadd);
 	else if (direction==2)
-		info->addr2 = g_list_append(info->addr2, vadd);
-	
+		info->addr2 = g_list_append(info->addr2, vadd);	
+
 	return info;
 }
 
@@ -763,7 +524,7 @@ static int
 packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const void *data)
 {
 	struct _sctp_info *sctp_info;
-	guint32 chunk_number = 0, tsnumber;
+	guint32 chunk_number = 0, tsnumber,framenumber;
 	sctp_tmp_info_t tmp_info;
 	sctp_assoc_info_t *info = NULL;
 	sctp_error_info_t *error = NULL;
@@ -786,6 +547,8 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 	sctp_info = (struct _sctp_info *) data;
 	max =0xFFFFFFFF;
 
+	framenumber=pinfo->fd->num;
+
 	type = sctp_info->ip_src.type;
 
 	if (type == AT_IPv4)
@@ -871,6 +634,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 			info->n_tvbs            = tmp_info.n_tvbs;
 			info->init              = FALSE;
 			info->initack           = FALSE;
+			info->check_address	= FALSE;
 			info->direction         = 0;
 			info = calc_checksum(sctp_info, info);
 			info->n_packets         = 1;
@@ -918,7 +682,9 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 				tsn  = g_malloc(sizeof(tsn_t));
 				sack = g_malloc(sizeof(tsn_t));
 				tsn->tsns  = NULL;
+				tsn->first_tsn = 0;
 				sack->tsns = NULL;
+				sack->first_tsn = 0;
 				sack->src.type=tsn->src.type = tmp_info.src.type;
 				sack->src.len=tsn->src.len   = tmp_info.src.len;
 				addr = g_malloc(tmp_info.src.len);
@@ -1012,6 +778,8 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 					sack = g_malloc(sizeof(tsn_t));
 					tsn->tsns  = NULL;
 					sack->tsns = NULL;
+					tsn->first_tsn = 0;
+					sack->first_tsn = 0;
 				}
 				for (chunk_number = 0; chunk_number < sctp_info->number_of_tvbs; chunk_number++)
 				{
@@ -1043,6 +811,8 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 							info->n_data_bytes_ep1+=length;
 							info->max_tsn1 = tsnumber;
 						}
+						if (tsn->first_tsn == 0)
+							tsn->first_tsn = tsnumber;
 						t_s_n = g_malloc(16);
 						tvb_memcpy(sctp_info->tvb[chunk_number], (guint8 *)(t_s_n),0, 16);
 						tsn->tsns = g_list_append(tsn->tsns, t_s_n);
@@ -1052,6 +822,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 						tsn_s->secs     = tsn->secs;
 						tsn_s->usecs    = tsn->usecs;
 						tsn_s->offset   = 0;
+						tsn_s->framenumber = framenumber;
 						tsn_s->length   = length-DATA_CHUNK_HEADER_LENGTH;
 						g_ptr_array_add(info->sort_tsn1, tsn_s);
 						info->n_array_tsn1++;
@@ -1065,6 +836,8 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 							info->max_tsn2 = tsnumber;
 						info->n_sack_chunks_ep2++;
 						length = tvb_get_ntohs(sctp_info->tvb[chunk_number], CHUNK_LENGTH_OFFSET);
+						if (sack->first_tsn == 0)
+							sack->first_tsn = tsnumber;
 						t_s_n = g_malloc(length);
 						tvb_memcpy(sctp_info->tvb[chunk_number], (guint8 *)(t_s_n),0, length);
 						sack->tsns = g_list_append(sack->tsns, t_s_n);
@@ -1074,6 +847,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 						tsn_s->secs     = tsn->secs;
 						tsn_s->usecs    = tsn->usecs;
 						tsn_s->offset   = 0;
+						tsn_s->framenumber = framenumber;
 						tsn_s->length   =  tvb_get_ntohl(sctp_info->tvb[chunk_number], SACK_CHUNK_ADV_REC_WINDOW_CREDIT_OFFSET);
 						if (tsn_s->length > info->max_window1)
 							info->max_window1 = tsn_s->length;
@@ -1109,13 +883,14 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 			{
 				error = g_malloc(sizeof(sctp_error_info_t));
 				error->frame_number = pinfo->fd->num;
-				strcpy(str,"");
-				strcpy(error->chunk_info,"");
+				str[0] = '\0';
+				error->chunk_info[0] = '\0';
 				if ((tvb_get_guint8(sctp_info->tvb[0],0)) == SCTP_INIT_CHUNK_ID)
-					strcpy(error->chunk_info, val_to_str(tvb_get_guint8(sctp_info->tvb[0],0),chunk_type_values,"Reserved"));
+					strncpy(error->chunk_info, val_to_str(tvb_get_guint8(sctp_info->tvb[0],0),chunk_type_values,"Reserved"), 200);
 				else
 					for (chunk_number = 0; chunk_number < sctp_info->number_of_tvbs; chunk_number++)
-						strcat(error->chunk_info, val_to_str(tvb_get_guint8(sctp_info->tvb[chunk_number],0),chunk_type_values,"Reserved"));
+						strncat(error->chunk_info, val_to_str(tvb_get_guint8(sctp_info->tvb[chunk_number],0),chunk_type_values,"Reserved"), 200 - strlen (error->chunk_info));
+				error->chunk_info[199] = '\0';
 				error->info_text = "INFOS";
 				info->error_info_list = g_list_append(info->error_info_list, error);
 			}
@@ -1132,7 +907,9 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 			tsn  = g_malloc(sizeof(tsn_t));
 			sack = g_malloc(sizeof(tsn_t));
 			tsn->tsns  = NULL;
+			tsn->first_tsn = 0;
 			sack->tsns = NULL;
+			sack->first_tsn = 0;
 			sack->src.type = tsn->src.type = tmp_info.src.type;
 			sack->src.len  = tsn->src.len = tmp_info.src.len;
 			addr = g_malloc(tmp_info.src.len);
@@ -1272,8 +1049,10 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 			{
 				sack = g_malloc(sizeof(tsn_t));
 				sack->tsns = NULL;
+				sack->first_tsn = 0;
 				tsn = g_malloc(sizeof(tsn_t));
 				tsn->tsns = NULL;
+				tsn->first_tsn = 0;
 			}
 			for (chunk_number = 0; chunk_number < sctp_info->number_of_tvbs; chunk_number++)
 			{
@@ -1298,6 +1077,8 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 				if ((tvb_get_guint8(sctp_info->tvb[chunk_number],0)) == SCTP_DATA_CHUNK_ID)
 				{
 					tsnumber = tvb_get_ntohl((sctp_info->tvb)[chunk_number], DATA_CHUNK_TSN_OFFSET);
+					if (tsn->first_tsn == 0)
+						tsn->first_tsn = tsnumber;
 					t_s_n = g_malloc(16);
 					tvb_memcpy(sctp_info->tvb[chunk_number], (guint8 *)(t_s_n),0, 16);
 					tsn->tsns = g_list_append(tsn->tsns, t_s_n);
@@ -1310,6 +1091,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 					tsn_s->secs  = tsn->secs;
 					tsn_s->usecs = tsn->usecs;
 					tsn_s->offset = 0;
+					tsn_s->framenumber = framenumber;
 					tsn_s->length = length;
 	
 					if (info->direction == 1)
@@ -1369,6 +1151,8 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 				{
 					tsnumber = tvb_get_ntohl((sctp_info->tvb)[chunk_number], SACK_CHUNK_CUMULATIVE_TSN_ACK_OFFSET);
 					length = tvb_get_ntohs(sctp_info->tvb[chunk_number], CHUNK_LENGTH_OFFSET);
+					if (sack->first_tsn == 0)
+						sack->first_tsn = tsnumber;
 					t_s_n = g_malloc(length);
 					tvb_memcpy(sctp_info->tvb[chunk_number], (guint8 *)(t_s_n),0, length);
 					sack->tsns = g_list_append(sack->tsns, t_s_n);
@@ -1378,6 +1162,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 					tsn_s->secs   = tsn->secs;
 					tsn_s->usecs  = tsn->usecs;
 					tsn_s->offset = 0;
+					tsn_s->framenumber = framenumber;
 					tsn_s->length = tvb_get_ntohl(sctp_info->tvb[chunk_number], SACK_CHUNK_ADV_REC_WINDOW_CREDIT_OFFSET);
 	
 	
