@@ -1,7 +1,7 @@
 /* win32-file-dlg.c
  * Native Windows file dialog routines
  *
- * $Id: win32-file-dlg.c 19413 2006-10-03 18:39:36Z ulfl $
+ * $Id: win32-file-dlg.c 21872 2007-05-22 00:33:46Z sfisher $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -109,7 +109,7 @@ static void range_update_dynamics(HWND sf_hwnd, packet_range_t *range);
 static void range_handle_wm_initdialog(HWND dlg_hwnd, packet_range_t *range);
 static void range_handle_wm_command(HWND dlg_hwnd, HWND ctrl, WPARAM w_param, packet_range_t *range);
 
-static TCHAR *build_file_type_list(gboolean save);
+static TCHAR *build_file_type_list(gboolean save, int *item_to_select);
 static int file_type_from_list_index(gboolean save, int index);
 
 static int            filetype;
@@ -133,48 +133,64 @@ char *dfilter_merge_str = NULL;
 
 gboolean
 win32_open_file (HWND h_wnd) {
-    static OPENFILENAME ofn;
+    OPENFILENAME *ofn;
     TCHAR  file_name[MAX_PATH] = _T("");
     int    err;
     char  *dirname;
     dfilter_t *dfp;
+	int    ofnsize;
 
-    /* XXX - Check for version and set OPENFILENAME_SIZE_VERSION_400
-       where appropriate */
-    ZeroMemory(&ofn, sizeof(ofn));
-#ifdef OPENFILENAME_SIZE_VERSION_400
-    of.lStructSize = OPENFILENAME_SIZE_VERSION_400;
-#else
-    ofn.lStructSize = sizeof(ofn);
-#endif
-    ofn.hwndOwner = h_wnd;
-    ofn.hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
-    ofn.lpstrFilter = build_file_type_list(FALSE /*save*/);
-    ofn.lpstrCustomFilter = NULL;
-    ofn.nMaxCustFilter = 0;
-    ofn.nFilterIndex = FILE_OPEN_DEFAULT;
-    ofn.lpstrFile = file_name;
-    ofn.nMaxFile = MAX_PATH;
-    ofn.lpstrFileTitle = NULL;
-    ofn.nMaxFileTitle = 0;
+	/* Remarks on OPENFILENAME_SIZE_VERSION_400:
+	 *
+	 * MSDN states that OPENFILENAME_SIZE_VERSION_400 should be used with
+	 * WINVER and _WIN32_WINNT >= 0x0500.
+	 * Unfortunately all these are compiler constants, while the underlying is a
+	 * problem based is a length check of the runtime version used.
+	 *
+	 * Instead of using OPENFILENAME_SIZE_VERSION_400, just malloc 
+	 * the OPENFILENAME size plus 12 bytes.
+	 * These 12 bytes are the difference between the two versions of this struct.
+	 *
+	 * Interestingly this fixes a bug, so the places bar e.g. "My Documents" 
+	 * is displayed - which wasn't the case with the former implementation.
+	 *
+	 * XXX - It's unclear if this length+12 works on all supported platforms,
+	 * NT4 is the question here. However, even if it fails, we must calculate
+	 * the length based on the runtime, not the compiler version anyway ...
+	 */
+    ofnsize = sizeof(OPENFILENAME) + 12;
+	ofn = g_malloc0(ofnsize);
+
+    ofn->lStructSize = ofnsize;
+    ofn->hwndOwner = h_wnd;
+    ofn->hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
+    ofn->lpstrFilter = build_file_type_list(FALSE /*!save*/, NULL);
+    ofn->lpstrCustomFilter = NULL;
+    ofn->nMaxCustFilter = 0;
+    ofn->nFilterIndex = FILE_OPEN_DEFAULT;
+    ofn->lpstrFile = file_name;
+    ofn->nMaxFile = MAX_PATH;
+    ofn->lpstrFileTitle = NULL;
+    ofn->nMaxFileTitle = 0;
     if (prefs.gui_fileopen_style == FO_STYLE_SPECIFIED && prefs.gui_fileopen_dir[0] != '\0') {
-	ofn.lpstrInitialDir = utf_8to16(prefs.gui_fileopen_dir);
+	ofn->lpstrInitialDir = utf_8to16(prefs.gui_fileopen_dir);
     } else {
-	ofn.lpstrInitialDir = NULL;
+	ofn->lpstrInitialDir = utf_8to16(get_last_open_dir());
     }
-    ofn.lpstrTitle = _T("Wireshark: Open Capture File");
-    ofn.Flags = OFN_ENABLESIZING | OFN_ENABLETEMPLATE | OFN_EXPLORER |
+    ofn->lpstrTitle = _T("Wireshark: Open Capture File");
+    ofn->Flags = OFN_ENABLESIZING | OFN_ENABLETEMPLATE | OFN_EXPLORER |
 	    OFN_NOCHANGEDIR | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY |
 	    OFN_ENABLEHOOK;
     if(topic_available(HELP_OPEN_WIN32_DIALOG)) {
-        ofn.Flags |= OFN_SHOWHELP;
+        ofn->Flags |= OFN_SHOWHELP;
     }
-    ofn.lpstrDefExt = NULL;
-    ofn.lpfnHook = open_file_hook_proc;
-    ofn.lpTemplateName = _T("WIRESHARK_OPENFILENAME_TEMPLATE");
+    ofn->lpstrDefExt = NULL;
+    ofn->lpfnHook = open_file_hook_proc;
+    ofn->lpTemplateName = _T("WIRESHARK_OPENFILENAME_TEMPLATE");
 
-    if (GetOpenFileName(&ofn)) {
-    g_free( (void *) ofn.lpstrFilter);
+    if (GetOpenFileName(ofn)) {
+    g_free( (void *) ofn->lpstrFilter);
+    g_free( (void *) ofn);
 
 	if (cf_open(&cfile, utf_16to8(file_name), FALSE, &err) != CF_OK) {
 	    return FALSE;
@@ -193,52 +209,53 @@ win32_open_file (HWND h_wnd) {
                 return TRUE;
                 break;
 	}
-    }
+	} else {
+	g_free( (void *) ofn->lpstrFilter);
+	g_free( (void *) ofn);
+	}
     return FALSE;
 }
 
 
 void
 win32_save_as_file(HWND h_wnd, action_after_save_e action_after_save, gpointer action_after_save_data) {
-    static OPENFILENAME ofn;
+    OPENFILENAME *ofn;
     TCHAR  file_name16[MAX_PATH] = _T("");
     GString *file_name8;
     gchar *file_last_dot;
     gchar *dirname;
+    int save_index;
+	int    ofnsize;
 
-    /* XXX - Check for version and set OPENFILENAME_SIZE_VERSION_400
-       where appropriate */
-    ZeroMemory(&ofn, sizeof(ofn));
-    ofn.lStructSize = sizeof(ofn);
-    ofn.hwndOwner = h_wnd;
-    ofn.hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
-    ofn.lpstrFilter = build_file_type_list(TRUE /*save*/);
-    ofn.lpstrCustomFilter = NULL;
-    ofn.nMaxCustFilter = 0;
-    ofn.nFilterIndex = FILE_SAVE_DEFAULT;
-    ofn.lpstrFile = file_name16;
-    ofn.nMaxFile = MAX_PATH;
-    ofn.lpstrFileTitle = NULL;
-    ofn.nMaxFileTitle = 0;
-    if (prefs.gui_fileopen_style == FO_STYLE_SPECIFIED && prefs.gui_fileopen_dir[0] != '\0') {
-	ofn.lpstrInitialDir = utf_8to16(prefs.gui_fileopen_dir);
-    } else {
-	ofn.lpstrInitialDir = NULL;
-    }
-    ofn.lpstrTitle = _T("Wireshark: Save file as");
-    ofn.Flags = OFN_ENABLESIZING | OFN_ENABLETEMPLATE | OFN_EXPLORER |
+	/* see OPENFILENAME comment in win32_open_file */
+    ofnsize = sizeof(OPENFILENAME) + 12;
+	ofn = g_malloc0(ofnsize);
+
+    ofn->lStructSize = ofnsize;
+    ofn->hwndOwner = h_wnd;
+    ofn->hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
+    ofn->lpstrFilter = build_file_type_list(TRUE /*save*/, &save_index);
+    ofn->lpstrCustomFilter = NULL;
+    ofn->nMaxCustFilter = 0;
+    ofn->nFilterIndex = save_index;
+    ofn->lpstrFile = file_name16;
+    ofn->nMaxFile = MAX_PATH;
+    ofn->lpstrFileTitle = NULL;
+    ofn->nMaxFileTitle = 0;
+	ofn->lpstrInitialDir = utf_8to16(get_last_open_dir());
+    ofn->lpstrTitle = _T("Wireshark: Save file as");
+    ofn->Flags = OFN_ENABLESIZING | OFN_ENABLETEMPLATE | OFN_EXPLORER |
 	    OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY |
 	    OFN_PATHMUSTEXIST | OFN_ENABLEHOOK;
     if(topic_available(HELP_SAVE_WIN32_DIALOG)) {
-        ofn.Flags |= OFN_SHOWHELP;
+        ofn->Flags |= OFN_SHOWHELP;
     }
-    ofn.lpstrDefExt = NULL;
-    ofn.lpfnHook = save_as_file_hook_proc;
-    ofn.lpTemplateName = _T("WIRESHARK_SAVEFILENAME_TEMPLATE");
+    ofn->lpstrDefExt = NULL;
+    ofn->lpfnHook = save_as_file_hook_proc;
+    ofn->lpTemplateName = _T("WIRESHARK_SAVEFILENAME_TEMPLATE");
 
-    if (GetSaveFileName(&ofn)) {
-    filetype = file_type_from_list_index(TRUE /*save*/, ofn.nFilterIndex);
-    g_free( (void *) ofn.lpstrFilter);
+    if (GetSaveFileName(ofn)) {
+    filetype = file_type_from_list_index(TRUE /*save*/, ofn->nFilterIndex);
 
     /* append the default file extension if there's none given by the user */
     /* (we expect a file extension to be at most 5 chars + the dot) */
@@ -261,6 +278,8 @@ win32_save_as_file(HWND h_wnd, action_after_save_e action_after_save, gpointer a
 	if (cf_save(&cfile, file_name8->str, &range, filetype, FALSE) != CF_OK) {
 	    /* The write failed.  Try again. */
         g_string_free(file_name8, TRUE /* free_segment */);
+		g_free( (void *) ofn->lpstrFilter);
+		g_free( (void *) ofn);
 	    win32_save_as_file(h_wnd, action_after_save, action_after_save_data);
 	    return;
 	}
@@ -303,12 +322,14 @@ win32_save_as_file(HWND h_wnd, action_after_save_e action_after_save, gpointer a
 	}
     }
     g_sf_hwnd = NULL;
+    g_free( (void *) ofn->lpstrFilter);
+    g_free( (void *) ofn);
 }
 
 
 void
 win32_merge_file (HWND h_wnd) {
-    static      OPENFILENAME ofn;
+    OPENFILENAME *ofn;
     TCHAR       file_name[MAX_PATH] = _T("");
     char       *dirname;
     cf_status_t merge_status;
@@ -316,42 +337,43 @@ win32_merge_file (HWND h_wnd) {
     int         err;
     char       *tmpname;
     dfilter_t *dfp;
+	int    ofnsize;
 
-    /* XXX - Check for temp file and prompt accordingly */
+	/* see OPENFILENAME comment in win32_open_file */
+    ofnsize = sizeof(OPENFILENAME) + 12;
+	ofn = g_malloc0(ofnsize);
 
-    /* XXX - Check for version and set OPENFILENAME_SIZE_VERSION_400
-       where appropriate */
-    ZeroMemory(&ofn, sizeof(ofn));
-    ofn.lStructSize = sizeof(ofn);
-    ofn.hwndOwner = h_wnd;
-    ofn.hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
-    ofn.lpstrFilter = build_file_type_list(FALSE /*save*/);
-    ofn.lpstrCustomFilter = NULL;
-    ofn.nMaxCustFilter = 0;
-    ofn.nFilterIndex = FILE_MERGE_DEFAULT;
-    ofn.lpstrFile = file_name;
-    ofn.nMaxFile = MAX_PATH;
-    ofn.lpstrFileTitle = NULL;
-    ofn.nMaxFileTitle = 0;
+    ofn->lStructSize = ofnsize;
+    ofn->hwndOwner = h_wnd;
+    ofn->hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
+    ofn->lpstrFilter = build_file_type_list(FALSE /*!save*/, NULL);
+    ofn->lpstrCustomFilter = NULL;
+    ofn->nMaxCustFilter = 0;
+    ofn->nFilterIndex = FILE_MERGE_DEFAULT;
+    ofn->lpstrFile = file_name;
+    ofn->nMaxFile = MAX_PATH;
+    ofn->lpstrFileTitle = NULL;
+    ofn->nMaxFileTitle = 0;
     if (prefs.gui_fileopen_style == FO_STYLE_SPECIFIED && prefs.gui_fileopen_dir[0] != '\0') {
-	ofn.lpstrInitialDir = utf_8to16(prefs.gui_fileopen_dir);
+	ofn->lpstrInitialDir = utf_8to16(prefs.gui_fileopen_dir);
     } else {
-	ofn.lpstrInitialDir = NULL;
+	ofn->lpstrInitialDir = utf_8to16(get_last_open_dir());
     }
-    ofn.lpstrTitle = _T("Wireshark: Merge with capture file");
-    ofn.Flags = OFN_ENABLESIZING | OFN_ENABLETEMPLATE | OFN_EXPLORER |
+    ofn->lpstrTitle = _T("Wireshark: Merge with capture file");
+    ofn->Flags = OFN_ENABLESIZING | OFN_ENABLETEMPLATE | OFN_EXPLORER |
 	    OFN_NOCHANGEDIR | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY |
 	    OFN_ENABLEHOOK;
     if(topic_available(HELP_MERGE_WIN32_DIALOG)) {
-        ofn.Flags |= OFN_SHOWHELP;
+        ofn->Flags |= OFN_SHOWHELP;
     }
-    ofn.lpstrDefExt = NULL;
-    ofn.lpfnHook = merge_file_hook_proc;
-    ofn.lpTemplateName = _T("WIRESHARK_MERGEFILENAME_TEMPLATE");
+    ofn->lpstrDefExt = NULL;
+    ofn->lpfnHook = merge_file_hook_proc;
+    ofn->lpTemplateName = _T("WIRESHARK_MERGEFILENAME_TEMPLATE");
 
-    if (GetOpenFileName(&ofn)) {
+    if (GetOpenFileName(ofn)) {
 	filetype = cfile.cd_t;
-    g_free( (void *) ofn.lpstrFilter);
+    g_free( (void *) ofn->lpstrFilter);
+    g_free( (void *) ofn);
 
 	/* merge or append the two files */
 
@@ -359,8 +381,8 @@ win32_merge_file (HWND h_wnd) {
 	switch (merge_action) {
 	    case merge_append:
 		/* append file */
-		in_filenames[0] = utf_16to8(file_name);
-		in_filenames[1] = cfile.filename;
+		in_filenames[0] = cfile.filename;
+		in_filenames[1] = utf_16to8(file_name);
 		merge_status = cf_merge_files(&tmpname, 2, in_filenames, filetype, TRUE);
 		break;
 	    case merge_chrono:
@@ -371,9 +393,9 @@ win32_merge_file (HWND h_wnd) {
 		break;
 	    case merge_prepend:
 		/* prepend file */
-		in_filenames[0] = cfile.filename;
-		in_filenames[1] = utf_16to8(file_name);
-                merge_status = cf_merge_files(&tmpname, 2, in_filenames, filetype, TRUE);
+		in_filenames[0] = utf_16to8(file_name);
+		in_filenames[1] = cfile.filename;
+		merge_status = cf_merge_files(&tmpname, 2, in_filenames, filetype, TRUE);
 		break;
 	    default:
 		g_assert_not_reached();
@@ -411,45 +433,46 @@ win32_merge_file (HWND h_wnd) {
             case CF_READ_ABORTED:
                 break;
         }
-    }
+	} else {
+	g_free( (void *) ofn->lpstrFilter);
+	g_free( (void *) ofn);
+	}
 }
 
 void
 win32_export_file(HWND h_wnd, export_type_e export_type) {
-    static            OPENFILENAME ofn;
+    OPENFILENAME     *ofn;
     TCHAR             file_name[MAX_PATH] = _T("");
     char             *dirname;
     cf_print_status_t status;
+	int    ofnsize;
 
-    /* XXX - Check for version and set OPENFILENAME_SIZE_VERSION_400
-       where appropriate */
-    ZeroMemory(&ofn, sizeof(ofn));
-    ofn.lStructSize = sizeof(ofn);
-    ofn.hwndOwner = h_wnd;
-    ofn.hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
-    ofn.lpstrFilter = FILE_TYPES_EXPORT;
-    ofn.lpstrCustomFilter = NULL;
-    ofn.nMaxCustFilter = 0;
-    ofn.nFilterIndex = export_type;
-    ofn.lpstrFile = file_name;
-    ofn.nMaxFile = MAX_PATH;
-    ofn.lpstrFileTitle = NULL;
-    ofn.nMaxFileTitle = 0;
-    if (prefs.gui_fileopen_style == FO_STYLE_SPECIFIED && prefs.gui_fileopen_dir[0] != '\0') {
-	ofn.lpstrInitialDir = utf_8to16(prefs.gui_fileopen_dir);
-    } else {
-	ofn.lpstrInitialDir = NULL;
-    }
-    ofn.lpstrTitle = _T("Wireshark: Export File");
-    ofn.Flags = OFN_ENABLESIZING | OFN_ENABLETEMPLATE | OFN_EXPLORER |
+	/* see OPENFILENAME comment in win32_open_file */
+    ofnsize = sizeof(OPENFILENAME) + 12;
+	ofn = g_malloc0(ofnsize);
+
+    ofn->lStructSize = ofnsize;
+    ofn->hwndOwner = h_wnd;
+    ofn->hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
+    ofn->lpstrFilter = FILE_TYPES_EXPORT;
+    ofn->lpstrCustomFilter = NULL;
+    ofn->nMaxCustFilter = 0;
+    ofn->nFilterIndex = export_type;
+    ofn->lpstrFile = file_name;
+    ofn->nMaxFile = MAX_PATH;
+    ofn->lpstrFileTitle = NULL;
+    ofn->nMaxFileTitle = 0;
+	ofn->lpstrInitialDir = utf_8to16(get_last_open_dir());
+    ofn->lpstrTitle = _T("Wireshark: Export File");
+    ofn->Flags = OFN_ENABLESIZING | OFN_ENABLETEMPLATE | OFN_EXPLORER |
 	    OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY |
 	    OFN_PATHMUSTEXIST | OFN_ENABLEHOOK;
     if(topic_available(HELP_EXPORT_FILE_WIN32_DIALOG)) {
-        ofn.Flags |= OFN_SHOWHELP;
+        ofn->Flags |= OFN_SHOWHELP;
     }
-    ofn.lpstrDefExt = NULL;
-    ofn.lpfnHook = export_file_hook_proc;
-    ofn.lpTemplateName = _T("WIRESHARK_EXPORTFILENAME_TEMPLATE");
+    ofn->lpstrDefExt = NULL;
+    ofn->lpfnHook = export_file_hook_proc;
+    ofn->lpTemplateName = _T("WIRESHARK_EXPORTFILENAME_TEMPLATE");
 
     /* Fill in our print (and export) args */
 
@@ -461,13 +484,14 @@ win32_export_file(HWND h_wnd, export_type_e export_type) {
     print_args.print_hex           = FALSE;
     print_args.print_formfeed      = FALSE;
 
-    if (GetSaveFileName(&ofn)) {
+    if (GetSaveFileName(ofn)) {
 	print_args.file = utf_16to8(file_name);
-	switch (ofn.nFilterIndex) {
+	switch (ofn->nFilterIndex) {
 	    case export_type_text:	/* Text */
 		print_args.stream = print_stream_text_new(TRUE, print_args.file);
 		if (print_args.stream == NULL) {
 		    open_failure_alert_box(print_args.file, errno, TRUE);
+		    g_free( (void *) ofn);
 		    return;
 		}
 		status = cf_print_packets(&cfile, &print_args);
@@ -476,6 +500,7 @@ win32_export_file(HWND h_wnd, export_type_e export_type) {
 		print_args.stream = print_stream_ps_new(TRUE, print_args.file);
 		if (print_args.stream == NULL) {
 		    open_failure_alert_box(print_args.file, errno, TRUE);
+		    g_free( (void *) ofn);
 		    return;
 		}
 		status = cf_print_packets(&cfile, &print_args);
@@ -490,6 +515,7 @@ win32_export_file(HWND h_wnd, export_type_e export_type) {
 		status = cf_write_pdml_packets(&cfile, &print_args);
 		break;
 	    default:
+	    g_free( (void *) ofn);
 		return;
 	}
 
@@ -506,17 +532,20 @@ win32_export_file(HWND h_wnd, export_type_e export_type) {
 	/* Save the directory name for future file dialogs. */
 	dirname = get_dirname(utf_16to8(file_name));  /* Overwrites cf_name */
 	set_last_open_dir(dirname);
-    }
+	}
+
+	g_free( (void *) ofn);
 }
 
 void
 win32_export_raw_file(HWND h_wnd) {
-    static        OPENFILENAME ofn;
+    OPENFILENAME *ofn;
     TCHAR         file_name[MAX_PATH] = _T("");
     char         *dirname;
     const guint8 *data_p = NULL;
     const char   *file = NULL;
     int           fd;
+	int           ofnsize;
 
     if (!cfile.finfo_selected) {
 	/* This shouldn't happen */
@@ -524,41 +553,41 @@ win32_export_raw_file(HWND h_wnd) {
 	return;
     }
 
-    ZeroMemory(&ofn, sizeof(ofn));
-    ofn.lStructSize = sizeof(ofn);
-    ofn.hwndOwner = h_wnd;
-    ofn.hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
-    ofn.lpstrFilter = FILE_TYPES_RAW;
-    ofn.lpstrCustomFilter = NULL;
-    ofn.nMaxCustFilter = 0;
-    ofn.nFilterIndex = FILE_RAW_DEFAULT;
-    ofn.lpstrFile = file_name;
-    ofn.nMaxFile = MAX_PATH;
-    ofn.lpstrFileTitle = NULL;
-    ofn.nMaxFileTitle = 0;
-    if (prefs.gui_fileopen_style == FO_STYLE_SPECIFIED && prefs.gui_fileopen_dir[0] != '\0') {
-	ofn.lpstrInitialDir = utf_8to16(prefs.gui_fileopen_dir);
-    } else {
-	ofn.lpstrInitialDir = NULL;
-    }
-    ofn.lpstrTitle = _T("Wireshark: Export Raw Data");
-    ofn.Flags = OFN_ENABLESIZING | OFN_ENABLETEMPLATE | OFN_EXPLORER |
+	/* see OPENFILENAME comment in win32_open_file */
+    ofnsize = sizeof(OPENFILENAME) + 12;
+	ofn = g_malloc0(ofnsize);
+
+    ofn->lStructSize = ofnsize;
+    ofn->hwndOwner = h_wnd;
+    ofn->hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
+    ofn->lpstrFilter = FILE_TYPES_RAW;
+    ofn->lpstrCustomFilter = NULL;
+    ofn->nMaxCustFilter = 0;
+    ofn->nFilterIndex = FILE_RAW_DEFAULT;
+    ofn->lpstrFile = file_name;
+    ofn->nMaxFile = MAX_PATH;
+    ofn->lpstrFileTitle = NULL;
+    ofn->nMaxFileTitle = 0;
+	ofn->lpstrInitialDir = utf_8to16(get_last_open_dir());
+    ofn->lpstrTitle = _T("Wireshark: Export Raw Data");
+    ofn->Flags = OFN_ENABLESIZING | OFN_ENABLETEMPLATE | OFN_EXPLORER |
 	    OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY |
 	    OFN_PATHMUSTEXIST | OFN_ENABLEHOOK;
     if(topic_available(HELP_EXPORT_BYTES_WIN32_DIALOG)) {
-        ofn.Flags |= OFN_SHOWHELP;
+        ofn->Flags |= OFN_SHOWHELP;
     }
-    ofn.lpstrDefExt = NULL;
-    ofn.lCustData = cfile.finfo_selected->length;
-    ofn.lpfnHook = export_raw_file_hook_proc;
-    ofn.lpTemplateName = _T("WIRESHARK_EXPORTRAWFILENAME_TEMPLATE");
+    ofn->lpstrDefExt = NULL;
+    ofn->lCustData = cfile.finfo_selected->length;
+    ofn->lpfnHook = export_raw_file_hook_proc;
+    ofn->lpTemplateName = _T("WIRESHARK_EXPORTRAWFILENAME_TEMPLATE");
 
     /*
      * XXX - The GTK+ code uses get_byte_view_data_and_length().  We just
      * grab the info from cfile.finfo_selected.  Which is more "correct"?
      */
 
-    if (GetSaveFileName(&ofn)) {
+    if (GetSaveFileName(ofn)) {
+		g_free( (void *) ofn);
 	data_p = tvb_get_ptr(cfile.finfo_selected->ds_tvb, 0, -1) +
 		cfile.finfo_selected->start;
         fd = open(utf_16to8(file_name), O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, 0666);
@@ -579,85 +608,101 @@ win32_export_raw_file(HWND h_wnd) {
 	/* Save the directory name for future file dialogs. */
 	dirname = get_dirname(utf_16to8(file_name));  /* Overwrites cf_name */
 	set_last_open_dir(dirname);
-    }
+	} else {
+		g_free( (void *) ofn);
+	}
 }
 
 void
 win32_export_color_file(HWND h_wnd, gpointer filter_list) {
-    static OPENFILENAME ofn;
+    OPENFILENAME *ofn;
     TCHAR  file_name[MAX_PATH] = _T("");
     gchar *dirname;
+	int    ofnsize;
 
-    ZeroMemory(&ofn, sizeof(ofn));
-    ofn.lStructSize = sizeof(ofn);
-    ofn.hwndOwner = h_wnd;
-    ofn.hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
-    ofn.lpstrFilter = FILE_TYPES_COLOR;
-    ofn.lpstrCustomFilter = NULL;
-    ofn.nMaxCustFilter = 0;
-    ofn.nFilterIndex = FILE_DEFAULT_COLOR;
-    ofn.lpstrFile = file_name;
-    ofn.nMaxFile = MAX_PATH;
-    ofn.lpstrFileTitle = NULL;
-    ofn.nMaxFileTitle = 0;
-    ofn.lpstrInitialDir = NULL;
-    ofn.lpstrTitle = _T("Wireshark: Export Color Filters");
-    ofn.Flags = OFN_ENABLESIZING | OFN_EXPLORER |
+	/* see OPENFILENAME comment in win32_open_file */
+    ofnsize = sizeof(OPENFILENAME) + 12;
+	ofn = g_malloc0(ofnsize);
+
+    ofn->lStructSize = ofnsize;
+    ofn->hwndOwner = h_wnd;
+    ofn->hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
+    ofn->lpstrFilter = FILE_TYPES_COLOR;
+    ofn->lpstrCustomFilter = NULL;
+    ofn->nMaxCustFilter = 0;
+    ofn->nFilterIndex = FILE_DEFAULT_COLOR;
+    ofn->lpstrFile = file_name;
+    ofn->nMaxFile = MAX_PATH;
+    ofn->lpstrFileTitle = NULL;
+    ofn->nMaxFileTitle = 0;
+    ofn->lpstrInitialDir = utf_8to16(get_last_open_dir());
+    ofn->lpstrTitle = _T("Wireshark: Export Color Filters");
+    ofn->Flags = OFN_ENABLESIZING | OFN_EXPLORER |
 	    OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY |
 	    OFN_PATHMUSTEXIST | OFN_ENABLEHOOK;
-    ofn.lpstrDefExt = NULL;
-    ofn.lpfnHook = NULL;
-    ofn.lpTemplateName = NULL;
+    ofn->lpstrDefExt = NULL;
+    ofn->lpfnHook = NULL;
+    ofn->lpTemplateName = NULL;
 
     filetype = cfile.cd_t;
 
     /* XXX - Support marked filters */
-    if (GetSaveFileName(&ofn)) {
+    if (GetSaveFileName(ofn)) {
+		g_free( (void *) ofn);
 	if (!color_filters_export(utf_16to8(file_name), filter_list, FALSE /* all filters */))
 	    return;
 
 	/* Save the directory name for future file dialogs. */
 	dirname = get_dirname(utf_16to8(file_name));  /* Overwrites cf_name */
 	set_last_open_dir(dirname);
-    }
+	} else {
+		g_free( (void *) ofn);
+	}
 }
 
 void
 win32_import_color_file(HWND h_wnd, gpointer color_filters) {
-    static OPENFILENAME ofn;
+    OPENFILENAME *ofn;
     TCHAR  file_name[MAX_PATH] = _T("");
     gchar *dirname;
+	int    ofnsize;
 
-    ZeroMemory(&ofn, sizeof(ofn));
-    ofn.lStructSize = sizeof(ofn);
-    ofn.hwndOwner = h_wnd;
-    ofn.hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
-    ofn.lpstrFilter = FILE_TYPES_COLOR;
-    ofn.lpstrCustomFilter = NULL;
-    ofn.nMaxCustFilter = 0;
-    ofn.nFilterIndex = FILE_DEFAULT_COLOR;
-    ofn.lpstrFile = file_name;
-    ofn.nMaxFile = MAX_PATH;
-    ofn.lpstrFileTitle = NULL;
-    ofn.nMaxFileTitle = 0;
-    ofn.lpstrInitialDir = NULL;
-    ofn.lpstrTitle = _T("Wireshark: Import Color Filters");
-    ofn.Flags = OFN_ENABLESIZING | OFN_EXPLORER |
+	/* see OPENFILENAME comment in win32_open_file */
+    ofnsize = sizeof(OPENFILENAME) + 12;
+	ofn = g_malloc0(ofnsize);
+
+    ofn->lStructSize = ofnsize;
+    ofn->hwndOwner = h_wnd;
+    ofn->hInstance = (HINSTANCE) GetWindowLong(h_wnd, GWL_HINSTANCE);
+    ofn->lpstrFilter = FILE_TYPES_COLOR;
+    ofn->lpstrCustomFilter = NULL;
+    ofn->nMaxCustFilter = 0;
+    ofn->nFilterIndex = FILE_DEFAULT_COLOR;
+    ofn->lpstrFile = file_name;
+    ofn->nMaxFile = MAX_PATH;
+    ofn->lpstrFileTitle = NULL;
+    ofn->nMaxFileTitle = 0;
+    ofn->lpstrInitialDir = utf_8to16(get_last_open_dir());
+    ofn->lpstrTitle = _T("Wireshark: Import Color Filters");
+    ofn->Flags = OFN_ENABLESIZING | OFN_EXPLORER |
 	    OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY |
 	    OFN_PATHMUSTEXIST | OFN_ENABLEHOOK;
-    ofn.lpstrDefExt = NULL;
-    ofn.lpfnHook = NULL;
-    ofn.lpTemplateName = NULL;
+    ofn->lpstrDefExt = NULL;
+    ofn->lpfnHook = NULL;
+    ofn->lpTemplateName = NULL;
 
     /* XXX - Support export limited to selected filters */
-    if (GetOpenFileName(&ofn)) {
+    if (GetOpenFileName(ofn)) {
+		g_free( (void *) ofn);
 	if (!color_filters_import(utf_16to8(file_name), color_filters))
 	    return;
 
 	/* Save the directory name for future file dialogs. */
 	dirname = get_dirname(utf_16to8(file_name));  /* Overwrites cf_name */
 	set_last_open_dir(dirname);
-    }
+	} else {
+		g_free( (void *) ofn);
+	}
 }
 
 
@@ -770,7 +815,7 @@ preview_set_filename(HWND of_hwnd, gchar *preview_file) {
     int         err = 0;
     gchar      *err_info;
     TCHAR       string_buff[PREVIEW_STR_MAX];
-    long        data_offset;
+    gint64      data_offset;
     guint       packet = 0;
     guint64     filesize;
     time_t      ti_time;
@@ -1107,10 +1152,9 @@ file_type_from_list_index(gboolean save, int index) {
 
 
 static TCHAR *
-build_file_type_list(gboolean save) {
+build_file_type_list(gboolean save, int *item_to_select) {
     int   ft;
     guint index;
-    guint item_to_select;
     GString* str = g_string_new("");
     TCHAR *str16;
     GArray* sa = g_array_new(FALSE /*zero_terminated*/, FALSE /*clear_*/,2 /*element_size*/);
@@ -1119,7 +1163,13 @@ build_file_type_list(gboolean save) {
 
     /* Default to the first supported file type, if the file's current
        type isn't supported. */
-    item_to_select = 0;
+    if(item_to_select) {
+        if(save) {
+            *item_to_select = FILE_SAVE_DEFAULT;
+        } else {
+            *item_to_select = FILE_OPEN_DEFAULT;
+        }
+    }
 
     /* append the "All Files" entry */
 	if (!save) {
@@ -1132,7 +1182,7 @@ build_file_type_list(gboolean save) {
     }
 
     /* Check all file types. */
-    index = 0;
+    index = 1;  /* the index is one based! */
     for (ft = 0; ft < WTAP_NUM_FILE_TYPES; ft++) {
 	if (save && (!packet_range_process_all(&range) || ft != cfile.cd_t)) {
 	    /* not all unfiltered packets or a different file type.  We have to use Wiretap. */
@@ -1155,9 +1205,9 @@ build_file_type_list(gboolean save) {
     sa = g_array_append_vals(sa, str16, strlen(str->str));
     sa = g_array_append_val(sa, zero);
 
-	if (ft == filetype) {
+	if (ft == cfile.cd_t && item_to_select != NULL) {
 	    /* Default to the same format as the file, if it's supported. */
-	    item_to_select = index;
+	    *item_to_select = index;
 	}
 	index++;
     }
