@@ -1,10 +1,14 @@
 #!/bin/sh
+#
+# $Id: win32-setup.sh 24357 2008-02-16 16:35:49Z etxrab $
 
 # This MUST be in the form
 #   http://anonsvn.wireshark.org/wireshark-win32-libs/tags/<date>/packages
 # in order to provide backward compatibility with older trees (e.g. a
 # previous release or an older SVN checkout).
-DOWNLOAD_PREFIX="http://anonsvn.wireshark.org/wireshark-win32-libs/tags/2007-01-19/packages"
+# Save previous tag.
+#DOWNLOAD_PREFIX="http://anonsvn.wireshark.org/wireshark-win32-libs/tags/2008-01-29/packages/"
+DOWNLOAD_PREFIX="http://anonsvn.wireshark.org/wireshark-win32-libs/tags/2008-02-16/packages/"
 
 # Set DOWNLOAD_PREFIX to /packages to test uploads before creating the tag.
 #DOWNLOAD_PREFIX="http://anonsvn.wireshark.org/wireshark-win32-libs/trunk/packages/"
@@ -19,7 +23,8 @@ err_exit () {
 usage () {
 	echo "Usage:"
 	echo "	$0 --appverify <appname> [<appname>] ..."
-	echo "	$0 --download <destination> <subdirectory> <package>"
+	echo "  $0 --libverify <destination> <subdirectory> <package>"
+	echo "	$0 --download  <destination> <subdirectory> <package>"
 	echo ""
 	exit 1
 }
@@ -47,6 +52,17 @@ case "$1" in
 		echo "	$APP: $APP_LOC $res"
 	done
 	;;
+--libverify)
+	if [ -z "$2" -o -z "$3" -o -z "$4" ] ; then
+		usage
+	fi
+	DEST_PATH=`cygpath --dos "$2"`
+	PACKAGE_PATH=$4
+	PACKAGE=`basename "$PACKAGE_PATH"`
+	if [ ! -e $DEST_PATH/$PACKAGE ] ; then 
+	    err_exit "Package $PACKAGE is needed but is apparently not downloaded; 'nmake -f ... setup' required ?"
+	fi
+        ;;  
 --download)
 	if [ -z "$2" -o -z "$3" -o -z "$4" ] ; then
 		usage
@@ -55,6 +71,7 @@ case "$1" in
 	DEST_SUBDIR=$3
 	PACKAGE_PATH=$4
 	PACKAGE=`basename "$PACKAGE_PATH"`
+	echo ""
 	echo "****** $PACKAGE ******"
 	if [ -z "$http_proxy" -a -z "$HTTP_PROXY" ] ; then
 		echo "No HTTP proxy specified (http_proxy and HTTP_PROXY are empty)."
@@ -64,6 +81,7 @@ case "$1" in
 		use_proxy="-Y on"
 		if [ -z "$http_proxy" ] ; then
 			echo "HTTP proxy ($HTTP_PROXY) has been specified and will be used."
+			export http_proxy=$HTTP_PROXY
 		else
 			echo "HTTP proxy ($http_proxy) has been specified and will be used."
 		fi
@@ -80,8 +98,11 @@ case "$1" in
 	echo "Extracting $DEST_PATH/$PACKAGE into $DEST_PATH/$DEST_SUBDIR"
 	unzip -oq "$DEST_PATH/$PACKAGE" ||
 		err_exit "Couldn't unpack $DEST_PATH/$PACKAGE"
-	echo "Verifying that the DLLs in $DEST_PATH/$DEST_SUBDIR are executable."
-	for i in `/usr/bin/find $DEST_PATH/$DEST_SUBDIR -name \*\.dll` ; do
+	echo "Verifying that the DLLs and EXEs in $DEST_SUBDIR are executable."
+        # XX: Note that find will check *all* dlls/exes in DEST_SUBDIR and below
+        #     which may be more than those just unzipped depending upon DEST_SUBDIR.
+        #     This may cause extra repeated checks but will do no harm. 
+	for i in `/usr/bin/find . \( -name '*\.dll' -o -name '*\.exe' \)` ; do
 		if [ ! -x "$i" ] ; then
 			echo "Changing file permissions (add executable bit) to:"
 			echo "$i"
