@@ -1,6 +1,6 @@
 /* libpcap.c
  *
- * $Id: libpcap.c 23298 2007-10-28 18:52:45Z sake $
+ * $Id: libpcap.c 24119 2008-01-17 17:55:38Z morriss $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -24,10 +24,6 @@
 #include "config.h"
 #endif
 
-#ifdef HAVE_PCAP_H
-#include <pcap.h>
-#endif
-
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -38,13 +34,6 @@
 #include "erf.h"
 #include "libpcap.h"
 
-#ifdef HAVE_PCAP_H
-# ifdef HAVE_SYS_TYPES_H
-#   include <sys/types.h>
-# endif
-#include "wtap-capture.h"
-#endif
-
 /*
  * Various pseudo-headers that appear at the beginning of packet data.
  *
@@ -86,6 +75,16 @@
 #define MTP2_LINK_NUMBER_OFFSET		2	/* 2 bytes */
 #define MTP2_HDR_LEN			4	/* length of the header */
 
+/*
+ * A header containing additional SITA WAN information.
+ */
+#define SITA_FLAGS_OFFSET		0	/* 1 byte */
+#define SITA_SIGNALS_OFFSET		1	/* 1 byte */
+#define SITA_ERRORS1_OFFSET		2	/* 1 byte */
+#define SITA_ERRORS2_OFFSET		3	/* 1 byte */
+#define SITA_PROTO_OFFSET		4	/* 1 byte */
+#define SITA_HDR_LEN			5	/* length of the header */
+
 /*
  * The fake link-layer header of LAPD packets.
  */
@@ -138,6 +137,10 @@ static gboolean libpcap_get_mtp2_pseudoheader(const guint8 *mtp2_hdr,
     union wtap_pseudo_header *pseudo_header);
 static gboolean libpcap_read_mtp2_pseudoheader(FILE_T fh,
     union wtap_pseudo_header *pseudo_header, int *err, gchar **err_info);
+static gboolean libpcap_get_sita_pseudoheader(const guint8 *sita_phdr,
+    union wtap_pseudo_header *pseudo_header);
+static gboolean libpcap_read_sita_pseudoheader(FILE_T fh,
+    union wtap_pseudo_header *pseudo_header, int *err, gchar **err_info);
 static gboolean libpcap_get_lapd_pseudoheader(const guint8 *lapd_phdr,
     union wtap_pseudo_header *pseudo_header, int *err, gchar **err_info);
 static gboolean libpcap_read_lapd_pseudoheader(FILE_T fh,
@@ -362,6 +365,7 @@ static const struct {
 	{ 139,		WTAP_ENCAP_MTP2_WITH_PHDR },
 	{ 140,		WTAP_ENCAP_MTP2 },
 	{ 141,		WTAP_ENCAP_MTP3 },
+	{ 142,		WTAP_ENCAP_SCCP },
 	{ 143,		WTAP_ENCAP_DOCSIS },
 	{ 144,		WTAP_ENCAP_IRDA },	/* IrDA capture */
 
@@ -440,6 +444,8 @@ static const struct {
 	{ 189, 		WTAP_ENCAP_USB_LINUX },
         /* Per-Packet Information header */
         { 192,          WTAP_ENCAP_PPI },
+	/* SITA File Encapsulation */
+	{ 196,	        WTAP_ENCAP_SITA },
 	/* Endace Record File Encapsulation */
 	{ 197,	        WTAP_ENCAP_ERF },
 	/* Bluetooth HCI UART transport (part H:4) frames, like hcidump */
@@ -1382,6 +1388,29 @@ static gboolean libpcap_read(wtap *wth, int *err, gchar **err_info,
 		wth->data_offset += LAPD_SLL_LEN;
 		break;
 
+	case WTAP_ENCAP_SITA:
+		if (packet_size < SITA_HDR_LEN) {
+			/*
+			 * Uh-oh, the packet isn't big enough to even
+			 * have a pseudo-header.
+			 */
+			*err = WTAP_ERR_BAD_RECORD;
+			*err_info = g_strdup_printf("libpcap: SITA file has a %u-byte packet, too small to have even a SITA pseudo-header\n",
+			    packet_size);
+			return FALSE;
+		}
+		if (!libpcap_read_sita_pseudoheader(wth->fh, &wth->pseudo_header,
+		    err, err_info))
+			return FALSE;	/* Read error */
+
+		/*
+		 * Don't count the pseudo-header as part of the packet.
+		 */
+		orig_size -= SITA_HDR_LEN;
+		packet_size -= SITA_HDR_LEN;
+		wth->data_offset += SITA_HDR_LEN;
+		break;
+
 	case WTAP_ENCAP_USB_LINUX:
 		if (packet_size < sizeof (struct linux_usb_phdr)) {
 			/*
@@ -1389,7 +1418,7 @@ static gboolean libpcap_read(wtap *wth, int *err, gchar **err_info,
 			 * have a pseudo-header.
 			 */
 			*err = WTAP_ERR_BAD_RECORD;
-			*err_info = g_strdup_printf("libpcap: Linux USB file has a %u-byte packet, too small to have even a LAPD pseudo-header\n",
+			*err_info = g_strdup_printf("libpcap: Linux USB file has a %u-byte packet, too small to have even a Linux USB pseudo-header\n",
 			    packet_size);
 			return FALSE;
 		}
@@ -1587,6 +1616,14 @@ libpcap_seek_read(wtap *wth, gint64 seek_off,
 		}
 		break;
 
+	case WTAP_ENCAP_SITA:
+		if (!libpcap_read_sita_pseudoheader(wth->random_fh, pseudo_header,
+		    err, err_info)) {
+			/* Read error */
+			return FALSE;
+		}
+		break;
+
 	case WTAP_ENCAP_USB_LINUX:
 		if (!libpcap_read_linux_usb_pseudoheader(wth, wth->random_fh,
 		    pseudo_header, err))
@@ -1997,6 +2034,36 @@ libpcap_read_lapd_pseudoheader(FILE_T fh, union wtap_pseudo_header *pseudo_heade
 	    err_info);
 }
 
+static gboolean
+libpcap_get_sita_pseudoheader(const guint8 *sita_phdr,
+    union wtap_pseudo_header *pseudo_header)
+{
+	pseudo_header->sita.flags   = sita_phdr[SITA_FLAGS_OFFSET];
+	pseudo_header->sita.signals = sita_phdr[SITA_SIGNALS_OFFSET];
+	pseudo_header->sita.errors1 = sita_phdr[SITA_ERRORS1_OFFSET];
+	pseudo_header->sita.errors2 = sita_phdr[SITA_ERRORS2_OFFSET];
+	pseudo_header->sita.proto   = sita_phdr[SITA_PROTO_OFFSET];
+	return TRUE;
+}
+
+static gboolean
+libpcap_read_sita_pseudoheader(FILE_T fh, union wtap_pseudo_header *pseudo_header, int *err, gchar **err_info _U_)
+{
+	guint8	sita_phdr[SITA_HDR_LEN];
+	int	bytes_read;
+
+	errno = WTAP_ERR_CANT_READ;
+	bytes_read = file_read(sita_phdr, 1, SITA_HDR_LEN, fh);
+	if (bytes_read != SITA_HDR_LEN) {
+		*err = file_error(fh);
+		if (*err == 0)
+			*err = WTAP_ERR_SHORT_READ;
+		return FALSE;
+	}
+
+	return libpcap_get_sita_pseudoheader(sita_phdr, pseudo_header);
+}
+
 static void
 libpcap_swap_linux_usb_pseudoheader(struct linux_usb_phdr *phdr)
 {
@@ -2265,206 +2332,6 @@ static int wtap_wtap_encap_to_pcap_encap(int encap)
 	return -1;
 }
 
-#ifdef HAVE_PCAP_H
-/*
- * Given a Wiretap encapsulation type, and raw packet data and the packet
- * header from libpcap, process any pseudo-header in the packet,
- * fill in the Wiretap packet header, and return a pointer to the
- * beginning of the non-pseudo-header data in the packet.
- */
-const guchar *
-wtap_process_pcap_packet(gint linktype, const struct pcap_pkthdr *phdr,
-    const guchar *pd, union wtap_pseudo_header *pseudo_header,
-    struct wtap_pkthdr *whdr, int *err)
-{
-  guint size;
-
-	/* "phdr->ts" may not necessarily be a "struct timeval" - it may
-	   be a "struct bpf_timeval", with member sizes wired to 32
-	   bits - and we may go that way ourselves in the future, so
-	   copy the members individually. */
-	whdr->ts.secs = phdr->ts.tv_sec;
-	whdr->ts.nsecs = phdr->ts.tv_usec * 1000;
-	whdr->caplen = phdr->caplen;
-	whdr->len = phdr->len;
-	whdr->pkt_encap = linktype;
-
-	/*
-	 * If this is an ATM packet, the first four bytes are the
-	 * direction of the packet (transmit/receive), the VPI, and
-	 * the VCI; read them and generate the pseudo-header from
-	 * them.
-	 */
-	switch (linktype) {
-
-	case WTAP_ENCAP_ATM_PDUS:
-		if (whdr->caplen < SUNATM_LEN) {
-			/*
-			 * Uh-oh, the packet isn't big enough to even
-			 * have a pseudo-header.
-			 */
-			g_message("libpcap: SunATM capture has a %u-byte packet, too small to have even an ATM pseudo-header\n",
-			    whdr->caplen);
-			*err = WTAP_ERR_BAD_RECORD;
-			return NULL;
-		}
-		libpcap_get_sunatm_pseudoheader(pd, pseudo_header);
-
-		/*
-		 * Don't count the pseudo-header as part of the packet.
-		 */
-		whdr->len -= SUNATM_LEN;
-		whdr->caplen -= SUNATM_LEN;
-		pd += SUNATM_LEN;
-
-		/*
-		 * If this is ATM LANE traffic, try to guess what type of
-		 * LANE traffic it is based on the packet contents.
-		 */
-		if (pseudo_header->atm.type == TRAF_LANE)
-			atm_guess_lane_type(pd, whdr->caplen, pseudo_header);
-		break;
-
-	case WTAP_ENCAP_IRDA:
-		if (whdr->caplen < IRDA_SLL_LEN) {
-			/*
-			 * Uh-oh, the packet isn't big enough to even
-			 * have a pseudo-header.
-			 */
-			g_message("libpcap: IrDA capture has a %u-byte packet, too small to have even an IrDA pseudo-header\n",
-			    whdr->caplen);
-			*err = WTAP_ERR_BAD_RECORD;
-			return NULL;
-		}
-		if (!libpcap_get_irda_pseudoheader(pd, pseudo_header, err, NULL))
-			return NULL;
-
-		/*
-		 * Don't count the pseudo-header as part of the packet.
-		 */
-		whdr->len -= IRDA_SLL_LEN;
-		whdr->caplen -= IRDA_SLL_LEN;
-		pd += IRDA_SLL_LEN;
-		break;
-
-	case WTAP_ENCAP_MTP2_WITH_PHDR:
-		if (whdr->caplen < MTP2_HDR_LEN) {
-			/*
-			 * Uh-oh, the packet isn't big enough to even
-			 * have a pseudo-header.
-			 */
-			g_message("libpcap: MTP2 capture has a %u-byte packet, too small to have even an MTP2 pseudo-header\n",
-			    whdr->caplen);
-			*err = WTAP_ERR_BAD_RECORD;
-			return NULL;
-		}
-		if (!libpcap_get_mtp2_pseudoheader(pd, pseudo_header))
-			return NULL;
-
-		/*
-		 * Don't count the pseudo-header as part of the packet.
-		 */
-		whdr->len -= MTP2_HDR_LEN;
-		whdr->caplen -= MTP2_HDR_LEN;
-		pd += MTP2_HDR_LEN;
-		break;
-
-	case WTAP_ENCAP_LINUX_LAPD:
-		if (whdr->caplen < LAPD_SLL_LEN) {
-			/*
-			 * Uh-oh, the packet isn't big enough to even
-			 * have a pseudo-header.
-			 */
-			g_message("libpcap: LAPD capture has a %u-byte packet, too small to have even an LAPD pseudo-header\n",
-			    whdr->caplen);
-			*err = WTAP_ERR_BAD_RECORD;
-			return NULL;
-		}
-		if (!libpcap_get_lapd_pseudoheader(pd, pseudo_header, err, NULL))
-			return NULL;
-
-		/*
-		 * Don't count the pseudo-header as part of the packet.
-		 */
-		whdr->len -= LAPD_SLL_LEN;
-		whdr->caplen -= LAPD_SLL_LEN;
-		pd += LAPD_SLL_LEN;
-		break;
-
-	case WTAP_ENCAP_USB_LINUX:
-		if (whdr->caplen < sizeof (struct linux_usb_phdr)) {
-			/*
-			 * Uh-oh, the packet isn't big enough to even
-			 * have a pseudo-header.
-			 */
-			g_message("libpcap: Linux USB file has a %u-byte packet, too small to have even a LAPD pseudo-header\n",
-			    whdr->caplen);
-			*err = WTAP_ERR_BAD_RECORD;
-			return NULL;
-		}
-		/*
-		* Don't count the pseudo-header as part of the packet.
-		*/
-		whdr->len -= sizeof (struct linux_usb_phdr);
-		whdr->caplen -= sizeof (struct linux_usb_phdr);
-		pd += sizeof (struct linux_usb_phdr);
-		break;
-
-	case WTAP_ENCAP_ERF:
-		if (whdr->caplen < sizeof(struct erf_phdr) ) {
-			/*
-			 * Uh-oh, the packet isn't big enough to even
-			 * have a pseudo-header.
-			 */
-			g_message("libpcap: ERF capture has a %u-byte packet, too small to have even an ERF pseudo-header\n",
-			    whdr->caplen);
-			*err = WTAP_ERR_BAD_RECORD;
-			return NULL;
-		}
-		if (!libpcap_get_erf_pseudoheader(pd, whdr, pseudo_header))
-			return NULL;
-
-		/*
-		 * Don't count the pseudo-header as part of the packet.
-		 */
-		whdr->len -= sizeof(struct erf_phdr);
-		whdr->caplen -= sizeof(struct erf_phdr);
-		pd += sizeof(struct erf_phdr);
-
-		if (!libpcap_get_erf_subheader(pd, pseudo_header, &size))
-		  return NULL;
-
-		/*
-		 * Don't count the pseudo-header as part of the packet.
-		 */
-		whdr->len -= size;
-		whdr->caplen -= size;
-		pd += size;
-		break;
-
-	case WTAP_ENCAP_BLUETOOTH_H4_WITH_PHDR:
-		if (whdr->caplen < sizeof (struct libpcap_bt_phdr)) {
-			/*
-			 * Uh-oh, the packet isn't big enough to even
-			 * have a pseudo-header.
-			 */
-			g_message("libpcap: bluetooth file has a %u-byte packet, too small to have even a pseudo-header\n",
-			    whdr->caplen);
-			*err = WTAP_ERR_BAD_RECORD;
-			return NULL;
-		}
-		/*
-		 * Don't count the pseudo-header as part of the packet.
-		 */
-		whdr->len -= sizeof (struct libpcap_bt_phdr);
-		whdr->caplen -= sizeof (struct libpcap_bt_phdr);
-		pd += sizeof (struct libpcap_bt_phdr);
-		break;
-	}
-	return pd;
-}
-#endif
-
 /* Returns 0 if we could write the specified encapsulation type,
    an error indication otherwise. */
 int libpcap_dump_can_write_encap(int encap)
@@ -2575,6 +2442,7 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 	guint8 irda_hdr[IRDA_SLL_LEN];
 	guint8 lapd_hdr[LAPD_SLL_LEN];
 	guint8 mtp2_hdr[MTP2_HDR_LEN];
+	guint8 sita_hdr[SITA_HDR_LEN];
 	guint8 erf_hdr[ sizeof(struct erf_mc_phdr)];
 	int hdrsize, size;
 
@@ -2596,6 +2464,10 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 		hdrsize = LAPD_SLL_LEN;
 		break;
 
+	case WTAP_ENCAP_SITA:
+		hdrsize = SITA_HDR_LEN;
+		break;
+
 	case WTAP_ENCAP_USB_LINUX:
 		hdrsize = sizeof (struct linux_usb_phdr);
 		break;
@@ -2813,6 +2685,27 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 		wdh->bytes_dumped += sizeof(lapd_hdr);
 		break;
 
+	case WTAP_ENCAP_SITA:
+		/*
+		 * Write the SITA header.
+		 */
+		memset(&sita_hdr, 0, sizeof(sita_hdr));
+		sita_hdr[SITA_FLAGS_OFFSET]   = pseudo_header->sita.flags;
+		sita_hdr[SITA_SIGNALS_OFFSET] = pseudo_header->sita.signals;
+		sita_hdr[SITA_ERRORS1_OFFSET] = pseudo_header->sita.errors1;
+		sita_hdr[SITA_ERRORS2_OFFSET] = pseudo_header->sita.errors2;
+		sita_hdr[SITA_PROTO_OFFSET]   = pseudo_header->sita.proto;
+		nwritten = fwrite(&sita_hdr, 1, sizeof(sita_hdr), wdh->fh);
+		if (nwritten != sizeof(sita_hdr)) {
+			if (nwritten == 0 && ferror(wdh->fh))
+				*err = errno;
+			else
+				*err = WTAP_ERR_SHORT_WRITE;
+			return FALSE;
+		}
+		wdh->bytes_dumped += sizeof(sita_hdr);
+		break;
+
 	case WTAP_ENCAP_USB_LINUX:
 		/*
 		 * Write out the pseudo-header; it has the same format
